#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.mozambique import Mozambique
from tests.common import CommonCountryTests


class TestMozambique(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Mozambique, years=range(1975, 2050))

    def test_holidays(self):
        for year in range(1975, 2050):
            self.assertHoliday(
                f"{year}-01-01",
                f"{year}-02-03",
                f"{year}-04-07",
                f"{year}-05-01",
                f"{year}-06-25",
                f"{year}-09-07",
                f"{year}-09-25",
                f"{year}-12-25",
            )

        self.assertNoHoliday(f"{year}-10-04" for year in range(1975, 1993))
        self.assertNoHolidayName("Dia da Paz e Reconciliação", range(1975, 1993))
        self.assertHoliday(f"{year}-10-04" for year in range(1993, 2050))

    def test_observed(self):
        dt = (
            "2011-05-02",
            "2011-09-26",
            "2011-12-26",
            "2012-01-02",
            "2013-02-04",
            "2013-04-08",
            "2014-09-08",
            "2015-10-05",
            "2016-05-02",
            "2016-09-26",
            "2016-12-26",
            "2017-01-02",
            "2017-06-26",
            "2019-02-04",
            "2019-04-08",
            "2020-10-05",
            "2022-05-02",
            "2022-09-26",
            "2022-12-26",
            "2023-01-02",
            "2023-06-26",
        )
        self.assertHoliday(dt)
        self.assertNoNonObservedHoliday(dt)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Dia da Fraternidade universal"),
            ("2023-01-02", "Dia da Fraternidade universal (ponte)"),
            ("2023-02-03", "Dia dos Heróis Moçambicanos"),
            ("2023-04-07", "Dia da Mulher Moçambicana"),
            ("2023-05-01", "Dia Internacional dos Trabalhadores"),
            ("2023-06-25", "Dia da Independência Nacional"),
            ("2023-06-26", "Dia da Independência Nacional (ponte)"),
            ("2023-09-07", "Dia da Vitória"),
            ("2023-09-25", "Dia das Forças Armadas de Libertação Nacional"),
            ("2023-10-04", "Dia da Paz e Reconciliação"),
            ("2023-12-25", "Dia da Família"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "International Fraternalism Day"),
            ("2023-01-02", "International Fraternalism Day (observed)"),
            ("2023-02-03", "Heroes' Day"),
            ("2023-04-07", "Women's Day"),
            ("2023-05-01", "International Workers' Day"),
            ("2023-06-25", "Independence Day"),
            ("2023-06-26", "Independence Day (observed)"),
            ("2023-09-07", "Victory Day"),
            ("2023-09-25", "Armed Forces Day"),
            ("2023-10-04", "Peace and Reconciliation Day"),
            ("2023-12-25", "Family Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2023-01-01", "День всесвітнього братерства"),
            ("2023-01-02", "День всесвітнього братерства (вихідний)"),
            ("2023-02-03", "День героїв Мозамбіку"),
            ("2023-04-07", "День жінок Мозамбіку"),
            ("2023-05-01", "Міжнародний день трудящих"),
            ("2023-06-25", "День національної незалежності"),
            ("2023-06-26", "День національної незалежності (вихідний)"),
            ("2023-09-07", "День Перемоги"),
            ("2023-09-25", "День Збройних сил національного визволення"),
            ("2023-10-04", "День миру та примирення"),
            ("2023-12-25", "День родини"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import re
import sys
from pathlib import Path

FIND_URL_PATTERN = re.compile(r'https?://[^\s<>"]+')
REPLACEMENTS = {
    # Space (U+0020)
    " ": " ",  # No-Break Space (U+00A0)
    " ": " ",  # En Space (U+2002)
    " ": " ",  # Em Space (U+2003)
    " ": " ",  # Three-Per-Em Space (U+2004)
    " ": " ",  # Four-Per-Em Space (U+2005)
    " ": " ",  # Six-Per-Em Space (U+2006)
    " ": " ",  # Figure Space (U+2007)
    " ": " ",  # Punctuation Space (U+2008)
    " ": " ",  # Thin Space (U+2009)
    " ": " ",  # Hair Space (U+200A)
    " ": " ",  # Narrow No-Break Space (U+202F)
    " ": " ",  # Medium Mathematical Space (U+205F)
    "⠀": " ",  # Braille Pattern Blank (U+2800)
    # Quotation Mark (U+0022)
    "“": '"',  # Left Double Quotation Mark (U+201C)
    "”": '"',  # Right Double Quotation Mark (U+201D)
    # Apostrophe (U+0027)
    # Note: Modifier Letter Turned Comma (U+02BB) is used in Tongan and Uzbek
    #       while Modifier Letter Apostrophe (U+02BC) is used in Ukrainian.
    "‘": "'",  # Left Single Quotation Mark (U+2018)
    "’": "'",  # Right Single Quotation Mark (U+2019)
    # Hyphen Minus (U+002D)
    "‐": "-",  # Hyphen (U+2010)
    "‑": "-",  # Non-Breaking Hyphen (U+2011)
    "‒": "-",  # Figure Dash (U+2012)
    "–": "-",  # En Dash (U+2013)
    "—": "-",  # Em Dash (U+2014)
    "―": "-",  # Horizontal Bar (U+2015)
    "−": "-",  # Minus Sign (U+2212)
    # Empty Characters
    # Note: Zero Width Non-Joiner (U+200C) and Zero Width Joiner (U+200D)
    #       are used in Hindi, Persian, and Urdu.
    "\u200b": "",  # Zero-Width Space (U+200B)
    "\u2060": "",  # Word Joiner (U+2060)
    "\ufeff": "",  # Zero Width No-Break Space / BOM (U+FEFF)
}
REPLACEMENT_TABLE = str.maketrans(REPLACEMENTS)
FORBIDDEN_CHARS = set(REPLACEMENTS)


class TextNormalizer:
    """Normalize text by replacing non-standard Unicode with ASCII."""

    @staticmethod
    def normalize(text: str) -> str:
        """Apply character replacements to a string while preserving URLs."""
        result: list[str] = []
        last_end = 0
        for match in FIND_URL_PATTERN.finditer(text):
            start, end = match.start(), match.end()
            result.append(text[last_end:start].translate(REPLACEMENT_TABLE))
            result.append(text[start:end])
            last_end = end
        result.append(text[last_end:].translate(REPLACEMENT_TABLE))
        return "".join(result)

    @staticmethod
    def fix_file(path: Path) -> bool:
        """Normalize a single file.

        Returns:
            True if file was modified, False otherwise.
        """
        if not path.is_file():
            return False
        try:
            # For Python >=3.13: original = path.read_text(encoding="utf-8", newline="")
            with path.open("r", encoding="utf-8", newline="") as f:
                original = f.read()
        except (OSError, UnicodeDecodeError) as e:
            print(f"Warning: Could not read {path}: {e}", file=sys.stderr)
            return False
        # Skip binary files
        if "\x00" in original:
            return False
        # Fast exit
        if FORBIDDEN_CHARS.isdisjoint(original):
            return False
        fixed = TextNormalizer.normalize(original)
        if fixed == original:
            return False
        try:
            path.write_text(fixed, encoding="utf-8", newline="")
        except OSError as e:
            print(f"Error writing {path}: {e}", file=sys.stderr)
            return False
        return True

    def run(self) -> int:
        """Run normalization over all files passed as command-line arguments."""
        changed = False
        for filename in sys.argv[1:]:
            path = Path(filename)
            if self.fix_file(path):
                print(f"Fixed: {path}")
                changed = True
        sys.exit(1 if changed else 0)


if __name__ == "__main__":
    TextNormalizer().run()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import JUN, JUL, SEP, OCT
from holidays.groups import ChristianHolidays, IslamicHolidays, InternationalHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SUN_TO_NEXT_MON


class Burundi(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays):
    """Burundi holidays.

    References:
        * [Décret n° 100/150 du 7 Juin 2021](https://web.archive.org/web/20240514031141/https://www.presidence.gov.bi/wp-content/uploads/2021/06/Liste-et-regime-des-jours-feries.pdf)
        * [Republic of Burundi's Ministry of Foreign Affairs Public Holidays List](https://web.archive.org/web/20250820080112/https://www.mae.gov.bi/en/official-holidays/)
        * <https://web.archive.org/web/20250820095115/https://www.timeanddate.com/holidays/burundi/2025>

    When it appears inappropriate for a public holiday listed in Article 1 to be observed because
    it coincides with a Sunday, it shall be postponed to the following working day, which shall
    accordingly be considered a public holiday and observed as such.
    """

    country = "BI"
    # %s (estimated).
    estimated_label = tr("%s (estimé)")
    default_language = "fr_BI"
    # %s (observed, estimated).
    observed_estimated_label = tr("%s (observé, estimé)")
    # %s (observed).
    observed_label = tr("%s (observé)")
    start_year = 1962
    supported_languages = ("en_US", "fr_BI")

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=BurundiIslamicHolidays, show_estimated=islamic_show_estimated
        )
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        dts_observed = set()

        # New Year's Day.
        dts_observed.add(self._add_new_years_day(tr("Jour de l'an")))

        if self._year >= 1992:
            # Unity Day.
            dts_observed.add(self._add_holiday_feb_5(tr("Fête de l'Unité")))

        if self._year >= 1995:
            dts_observed.add(
                self._add_holiday_apr_6(
                    # Commemoration of the Assassination of President Cyprien Ntaryamira.
                    tr("Commémoration de l'Assassinat du Président Cyprien Ntaryamira")
                )
            )

        # International Labor Day.
        dts_observed.add(self._add_labor_day(tr("Fête Internationale du Travail")))

        # Ascension Day.
        self._add_ascension_thursday(tr("Jour de l'Ascension"))

        if self._year >= 2022:
            dts_observed.add(
                self._add_holiday_jun_8(
                    # National Day of Patriotism and Commemoration of the Death of
                    # President Pierre Nkurunziza.
                    tr(
                        "Journée Nationale du Patriotisme et Commémoration de la Mort "
                        "du Président Pierre Nkurunziza"
                    )
                )
            )

        # Independence Day.
        dts_observed.add(self._add_holiday_jul_1(tr("Anniversaire de l'Indépendance")))

        # Assumption Day.
        dts_observed.add(self._add_assumption_of_mary_day(tr("Assomption")))

        dts_observed.add(
            self._add_holiday_oct_13(
                # Commemoration of the Assassination of National Hero, Prince Louis Rwagasore.
                tr("Commémoration de l'Assassinat du Héros National, le Prince Louis Rwagasore")
            )
        )
        if self._year >= 1994:
            dts_observed.add(
                self._add_holiday_oct_21(
                    # Commemoration of the Assassination of President Melchior Ndadaye.
                    tr("Commémoration de l'Assassinat du Président Melchior Ndadaye")
                )
            )

        # All Saints' Day.
        dts_observed.add(self._add_all_saints_day(tr("Toussaint")))

        # Christmas Day.
        dts_observed.add(self._add_christmas_day(tr("Noël")))

        # Eid al-Fitr.
        dts_observed.update(self._add_eid_al_fitr_day(tr("Aid-El-Fithr")))

        # Eid al-Adha.
        dts_observed.update(self._add_eid_al_adha_day(tr("Aid-El-Adha")))

        if self.observed:
            self._populate_observed(dts_observed)


class BI(Burundi):
    pass


class BDI(Burundi):
    pass


class BurundiIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2014, 2025)
    EID_AL_ADHA_DATES = {
        2014: (OCT, 5),
        2015: (SEP, 24),
        2016: (SEP, 13),
        2017: (SEP, 2),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2014, 2025)
    EID_AL_FITR_DATES = {
        2014: (JUL, 29),
        2015: (JUL, 18),
        2016: (JUL, 7),
        2017: (JUN, 26),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.bosnia_and_herzegovina import BosniaAndHerzegovina
from tests.common import CommonCountryTests


class TestBosniaAndHerzegovina(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        cls.full_range = range(2000, 2050)
        super().setUpClass(BosniaAndHerzegovina)

    def test_new_years_day(self):
        name = "Nova godina"
        name_observed = f"{name} (slobodan dan)"
        self.assertHolidayName(
            name,
            (f"{year}-01-01" for year in self.full_range),
            (f"{year}-01-02" for year in self.full_range),
        )

        obs_dts = {
            "BIH": (
                "2006-01-03",
                "2012-01-03",
                "2017-01-03",
                "2023-01-03",
            ),
            "BRC": (
                "2000-01-03",
                "2005-01-03",
                "2006-01-03",
                "2011-01-03",
                "2012-01-03",
                "2017-01-03",
                "2022-01-03",
                "2023-01-03",
            ),
            "SRP": (
                "2000-01-03",
                "2005-01-03",
                "2011-01-03",
                "2022-01-03",
            ),
        }
        for subdiv, holidays in self.subdiv_holidays.items():
            self.assertHolidayName(name_observed, holidays, obs_dts[subdiv])
            self.assertNoNonObservedHoliday(
                self.subdiv_holidays_non_observed[subdiv], obs_dts[subdiv]
            )

    def test_orthodox_christmas_eve(self):
        name = "Badnji dan (Pravoslavni)"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv != "BRC":
                self.assertHolidayName(
                    name, holidays, (f"{year}-01-06" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_orthodox_christmas(self):
        name = "Božić (Pravoslavni)"
        self.assertHolidayName(name, (f"{year}-01-07" for year in self.full_range))

        name_observed = f"{name} (slobodan dan)"
        obs_dts = (
            "2001-01-08",
            "2007-01-08",
            "2018-01-08",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "BRC":
                self.assertHolidayName(name_observed, holidays, obs_dts)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], obs_dts)
            else:
                self.assertNoHolidayName(name_observed, holidays, obs_dts)
        self.assertNoHolidayName(name_observed, obs_dts)

    def test_orthodox_new_year(self):
        name = "Pravoslavna Nova godina"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SRP":
                self.assertHolidayName(
                    name, holidays, (f"{year}-01-14" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_independence_day(self):
        name = "Dan nezavisnosti"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "BIH":
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-01" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_day_of_establishment_of_brcko_district(self):
        name = "Dan uspostavljanja Brčko distrikta"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "BRC":
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-08" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

        name_observed = f"{name} (slobodan dan)"
        obs_dts = (
            "2009-03-09",
            "2015-03-09",
            "2020-03-09",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "BRC":
                self.assertHolidayName(name_observed, holidays, obs_dts)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], obs_dts)
            else:
                self.assertNoHolidayName(name_observed, holidays, obs_dts)
        self.assertNoHolidayName(name_observed, obs_dts)

    def test_catholic_good_friday(self):
        name = "Veliki petak (Katolički)"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv != "BRC":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-04-10",
                    "2021-04-02",
                    "2022-04-15",
                    "2023-04-07",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_catholic_easter(self):
        name = "Uskrs (Katolički)"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv != "BRC":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-04-12",
                    "2021-04-04",
                    "2022-04-17",
                    "2023-04-09",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_catholic_monday(self):
        name = "Uskrsni ponedjeljak (Katolički)"
        dts = (
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, self.full_range)
        for holidays in self.subdiv_holidays.values():
            self.assertHolidayName(name, holidays, dts)
            self.assertHolidayName(name, holidays, self.full_range)

    def test_orthodox_good_friday(self):
        name = "Veliki petak (Pravoslavni)"
        dts = (
            "2020-04-17",
            "2021-04-30",
            "2022-04-22",
            "2023-04-14",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, self.full_range)
        for holidays in self.subdiv_holidays.values():
            self.assertHolidayName(name, holidays, dts)
            self.assertHolidayName(name, holidays, self.full_range)

    def test_orthodox_easter(self):
        name = "Vaskrs (Pravoslavni)"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv != "BRC":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-04-19",
                    "2021-05-02",
                    "2022-04-24",
                    "2023-04-16",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_orthodox_easter_monday(self):
        name = "Uskrsni ponedjeljak (Pravoslavni)"
        self.assertNoHolidayName(name)

        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv != "BRC":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-04-20",
                    "2021-05-03",
                    "2022-04-25",
                    "2023-04-17",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_labor_day(self):
        name = "Međunarodni praznik rada"
        name_observed = f"{name} (slobodan dan)"
        self.assertHolidayName(
            name,
            (f"{year}-05-01" for year in self.full_range),
            (f"{year}-05-02" for year in self.full_range),
        )

        obs_dts = {
            "BIH": (
                "2011-05-03",
                "2022-05-03",
            ),
            "BRC": (
                "2004-05-03",
                "2005-05-03",
                "2010-05-03",
                "2011-05-03",
                "2016-05-03",
                "2021-05-03",
                "2022-05-03",
            ),
            "SRP": (
                "2004-05-03",
                "2010-05-03",
                "2021-05-03",
            ),
        }
        for subdiv, holidays in self.subdiv_holidays.items():
            self.assertHolidayName(name_observed, holidays, obs_dts[subdiv])

    def test_victory_day(self):
        name = "Dan pobjede nad fašizmom"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv != "BRC":
                self.assertHolidayName(
                    name, holidays, (f"{year}-05-09" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_eid_al_fitr(self):
        name = "Ramazanski Bajram"
        dts_1 = (
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
        )
        dts_2 = (
            "2020-05-25",
            "2021-05-14",
            "2022-05-03",
            "2023-04-22",
        )
        self.assertHolidayName(name, dts_1)
        self.assertNoHolidayName(name, dts_2)

        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv != "BRC":
                self.assertHolidayName(name, holidays, dts_1, dts_2)
            else:
                self.assertHolidayName(name, holidays, dts_1)
                self.assertNoHolidayName(name, holidays, dts_2)

    def test_eid_al_adha(self):
        name = "Kurban Bajram"
        dts_1 = (
            "2006-01-10",
            "2006-12-31",
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
        )
        dts_2 = (
            "2007-01-01",
            "2020-08-01",
            "2021-07-21",
            "2022-07-10",
            "2023-06-29",
        )
        self.assertHolidayName(name, dts_1)
        self.assertNoHolidayName(name, dts_2)

        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv != "BRC":
                self.assertHolidayName(name, holidays, dts_1, dts_2)
            else:
                self.assertHolidayName(name, holidays, dts_1)
                self.assertNoHolidayName(name, holidays, dts_2)

    def test_dayton_agreement_day(self):
        name = "Dan uspostave Opšteg okvirnog sporazuma za mir u Bosni i Hercegovini"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SRP":
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-21" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_statehood_day(self):
        name = "Dan državnosti"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "BIH":
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-25" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_catholic_christmas_eve(self):
        name = "Badnji dan (Katolički)"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv != "BRC":
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-24" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_catholic_christmas(self):
        name = "Božić (Katolički)"
        name_observed = f"{name} (slobodan dan)"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))

        obs_dts = (
            "2005-12-26",
            "2011-12-26",
            "2016-12-26",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "BRC":
                self.assertHolidayName(name_observed, holidays, obs_dts)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], obs_dts)
            else:
                self.assertNoHoliday(holidays, obs_dts)
        self.assertNoHoliday(obs_dts)

    def test_2021(self):
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "Nova godina"),
            ("2021-01-02", "Nova godina"),
            ("2021-01-07", "Božić (Pravoslavni)"),
            ("2021-04-05", "Uskrsni ponedjeljak (Katolički)"),
            ("2021-04-30", "Veliki petak (Pravoslavni)"),
            ("2021-05-01", "Međunarodni praznik rada"),
            ("2021-05-02", "Međunarodni praznik rada"),
            ("2021-05-13", "Ramazanski Bajram"),
            ("2021-07-20", "Kurban Bajram"),
            ("2021-12-25", "Božić (Katolički)"),
        )

        self.assertSubdivBihHolidaysInYear(
            2021,
            ("2021-01-01", "Nova godina"),
            ("2021-01-02", "Nova godina"),
            ("2021-01-06", "Badnji dan (Pravoslavni)"),
            ("2021-01-07", "Božić (Pravoslavni)"),
            ("2021-03-01", "Dan nezavisnosti"),
            ("2021-04-02", "Veliki petak (Katolički)"),
            ("2021-04-04", "Uskrs (Katolički)"),
            ("2021-04-05", "Uskrsni ponedjeljak (Katolički)"),
            ("2021-04-30", "Veliki petak (Pravoslavni)"),
            ("2021-05-01", "Međunarodni praznik rada"),
            ("2021-05-02", "Međunarodni praznik rada; Vaskrs (Pravoslavni)"),
            ("2021-05-03", "Uskrsni ponedjeljak (Pravoslavni)"),
            ("2021-05-09", "Dan pobjede nad fašizmom"),
            ("2021-05-13", "Ramazanski Bajram"),
            ("2021-05-14", "Ramazanski Bajram"),
            ("2021-07-20", "Kurban Bajram"),
            ("2021-07-21", "Kurban Bajram"),
            ("2021-11-25", "Dan državnosti"),
            ("2021-12-24", "Badnji dan (Katolički)"),
            ("2021-12-25", "Božić (Katolički)"),
        )

        self.assertSubdivBrcHolidaysInYear(
            2021,
            ("2021-01-01", "Nova godina"),
            ("2021-01-02", "Nova godina"),
            ("2021-01-07", "Božić (Pravoslavni)"),
            ("2021-03-08", "Dan uspostavljanja Brčko distrikta"),
            ("2021-04-05", "Uskrsni ponedjeljak (Katolički)"),
            ("2021-04-30", "Veliki petak (Pravoslavni)"),
            ("2021-05-01", "Međunarodni praznik rada"),
            ("2021-05-02", "Međunarodni praznik rada"),
            ("2021-05-03", "Međunarodni praznik rada (slobodan dan)"),
            ("2021-05-13", "Ramazanski Bajram"),
            ("2021-07-20", "Kurban Bajram"),
            ("2021-12-25", "Božić (Katolički)"),
        )

        self.assertSubdivSrpHolidaysInYear(
            2021,
            ("2021-01-01", "Nova godina"),
            ("2021-01-02", "Nova godina"),
            ("2021-01-06", "Badnji dan (Pravoslavni)"),
            ("2021-01-07", "Božić (Pravoslavni)"),
            ("2021-01-14", "Pravoslavna Nova godina"),
            ("2021-04-02", "Veliki petak (Katolički)"),
            ("2021-04-04", "Uskrs (Katolički)"),
            ("2021-04-05", "Uskrsni ponedjeljak (Katolički)"),
            ("2021-04-30", "Veliki petak (Pravoslavni)"),
            ("2021-05-01", "Međunarodni praznik rada"),
            ("2021-05-02", "Međunarodni praznik rada; Vaskrs (Pravoslavni)"),
            (
                "2021-05-03",
                "Međunarodni praznik rada (slobodan dan); Uskrsni ponedjeljak (Pravoslavni)",
            ),
            ("2021-05-09", "Dan pobjede nad fašizmom"),
            ("2021-05-13", "Ramazanski Bajram"),
            ("2021-05-14", "Ramazanski Bajram"),
            ("2021-07-20", "Kurban Bajram"),
            ("2021-07-21", "Kurban Bajram"),
            ("2021-11-21", "Dan uspostave Opšteg okvirnog sporazuma za mir u Bosni i Hercegovini"),
            ("2021-12-24", "Badnji dan (Katolički)"),
            ("2021-12-25", "Božić (Katolički)"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Nova godina"),
            ("2022-01-02", "Nova godina"),
            ("2022-01-03", "Nova godina (slobodan dan)"),
            ("2022-01-06", "Badnji dan (Pravoslavni)"),
            ("2022-01-07", "Božić (Pravoslavni)"),
            ("2022-01-14", "Pravoslavna Nova godina"),
            ("2022-03-01", "Dan nezavisnosti"),
            ("2022-03-08", "Dan uspostavljanja Brčko distrikta"),
            ("2022-04-15", "Veliki petak (Katolički)"),
            ("2022-04-17", "Uskrs (Katolički)"),
            ("2022-04-18", "Uskrsni ponedjeljak (Katolički)"),
            ("2022-04-22", "Veliki petak (Pravoslavni)"),
            ("2022-04-24", "Vaskrs (Pravoslavni)"),
            ("2022-04-25", "Uskrsni ponedjeljak (Pravoslavni)"),
            ("2022-05-01", "Međunarodni praznik rada"),
            ("2022-05-02", "Međunarodni praznik rada; Ramazanski Bajram"),
            ("2022-05-03", "Međunarodni praznik rada (slobodan dan); Ramazanski Bajram"),
            ("2022-05-09", "Dan pobjede nad fašizmom"),
            ("2022-07-09", "Kurban Bajram"),
            ("2022-07-10", "Kurban Bajram"),
            ("2022-11-21", "Dan uspostave Opšteg okvirnog sporazuma za mir u Bosni i Hercegovini"),
            ("2022-11-25", "Dan državnosti"),
            ("2022-12-24", "Badnji dan (Katolički)"),
            ("2022-12-25", "Božić (Katolički)"),
            ("2022-12-26", "Božić (Katolički) (slobodan dan)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-02", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-01-06", "Orthodox Christmas Eve"),
            ("2022-01-07", "Orthodox Christmas Day"),
            ("2022-01-14", "Orthodox New Year"),
            ("2022-03-01", "Independence Day"),
            ("2022-03-08", "Day of establishment of Brčko District"),
            ("2022-04-15", "Catholic Good Friday"),
            ("2022-04-17", "Catholic Easter Sunday"),
            ("2022-04-18", "Catholic Easter Monday"),
            ("2022-04-22", "Orthodox Good Friday"),
            ("2022-04-24", "Orthodox Easter Sunday"),
            ("2022-04-25", "Orthodox Easter Monday"),
            ("2022-05-01", "International Labor Day"),
            ("2022-05-02", "Eid al-Fitr; International Labor Day"),
            ("2022-05-03", "Eid al-Fitr; International Labor Day (observed)"),
            ("2022-05-09", "Victory Day"),
            ("2022-07-09", "Eid al-Adha"),
            ("2022-07-10", "Eid al-Adha"),
            ("2022-11-21", "Dayton Agreement Day"),
            ("2022-11-25", "Statehood Day"),
            ("2022-12-24", "Catholic Christmas Eve"),
            ("2022-12-25", "Catholic Christmas Day"),
            ("2022-12-26", "Catholic Christmas Day (observed)"),
        )

    def test_l10n_sr(self):
        self.assertLocalizedHolidays(
            "sr",
            ("2022-01-01", "Нова година"),
            ("2022-01-02", "Нова година"),
            ("2022-01-03", "Нова година (слободан дан)"),
            ("2022-01-06", "Бадњи дан (Православни)"),
            ("2022-01-07", "Божић (Православни)"),
            ("2022-01-14", "Православна Нова година"),
            ("2022-03-01", "Дан независности"),
            ("2022-03-08", "Дан успостављања Брчко дистрикта"),
            ("2022-04-15", "Велики петак (Католички)"),
            ("2022-04-17", "Ускрс (Католички)"),
            ("2022-04-18", "Ускршњи понедељак (Католички)"),
            ("2022-04-22", "Велики петак (Православни)"),
            ("2022-04-24", "Васкрс (Православни)"),
            ("2022-04-25", "Ускршњи понедељак (Православни)"),
            ("2022-05-01", "Међународни празник рада"),
            ("2022-05-02", "Међународни празник рада; Рамазански Бајрам"),
            ("2022-05-03", "Међународни празник рада (слободан дан); Рамазански Бајрам"),
            ("2022-05-09", "Дан побједе над фашизмом"),
            ("2022-07-09", "Курбан Бајрам"),
            ("2022-07-10", "Курбан Бајрам"),
            ("2022-11-21", "Дан успоставе Општег оквирног споразума за мир у Босни и Херцеговини"),
            ("2022-11-25", "Дан државности"),
            ("2022-12-24", "Бадњи дан (Католички)"),
            ("2022-12-25", "Божић (Католички)"),
            ("2022-12-26", "Божић (Католички) (слободан дан)"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-02", "Новий рік"),
            ("2022-01-03", "Новий рік (вихідний)"),
            ("2022-01-06", "Святий вечір (православний)"),
            ("2022-01-07", "Різдво Христове (православне)"),
            ("2022-01-14", "Православний Новий рік"),
            ("2022-03-01", "День незалежності"),
            ("2022-03-08", "День заснування округу Брчко"),
            ("2022-04-15", "Страсна пʼятниця (католицька)"),
            ("2022-04-17", "Великдень (католицький)"),
            ("2022-04-18", "Великодній понеділок (католицький)"),
            ("2022-04-22", "Страсна пʼятниця (православна)"),
            ("2022-04-24", "Великдень (православний)"),
            ("2022-04-25", "Великодній понеділок (православний)"),
            ("2022-05-01", "Міжнародний день праці"),
            ("2022-05-02", "Міжнародний день праці; Рамазан-байрам"),
            ("2022-05-03", "Міжнародний день праці (вихідний); Рамазан-байрам"),
            ("2022-05-09", "День перемоги над фашизмом"),
            ("2022-07-09", "Курбан-байрам"),
            ("2022-07-10", "Курбан-байрам"),
            (
                "2022-11-21",
                "День укладання Загальної рамкової угоди про мир у Боснії та Герцеговині",
            ),
            ("2022-11-25", "День державності"),
            ("2022-12-24", "Святий вечір (католицький)"),
            ("2022-12-25", "Різдво Христове (католицьке)"),
            ("2022-12-26", "Різдво Христове (католицьке) (вихідний)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import GOVERNMENT, SCHOOL
from holidays.countries.egypt import Egypt
from tests.common import CommonCountryTests


class TestEgypt(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Egypt)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Egypt(categories=GOVERNMENT, years=range(self.start_year, 2018)))
        self.assertNoHolidays(Egypt(categories=SCHOOL, years=range(self.start_year, 2019)))

    def test_coptic_christmas_day(self):
        name = "عيد الميلاد المجيد"
        self.assertHolidayName(
            name, (f"{year}-01-07" for year in (*range(2002, 2022), *range(2024, self.end_year)))
        )
        self.assertNoHolidayName(name, range(self.start_year, 2002))

        obs_dts = (
            "2022-01-06",
            "2023-01-08",
        )
        self.assertHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_january_25_revolution(self):
        name_2009 = "عيد الشرطة"
        name_2012 = "ثورة ٢٥ يناير وعيد الشرطة"
        self.assertHolidayName(name_2009, (f"{year}-01-25" for year in range(2009, 2012)))
        self.assertNoHolidayName(
            name_2009, range(self.start_year, 2009), range(2012, self.end_year)
        )

        self.assertHolidayName(
            name_2012,
            "2020-01-25",
            "2024-01-25",
            "2025-01-25",
        )
        self.assertNonObservedHolidayName(
            name_2012, (f"{year}-01-25" for year in range(2012, self.end_year))
        )
        self.assertNoHolidayName(name_2012, range(self.start_year, 2012))

        obs_dts = (
            "2021-01-28",
            "2022-01-27",
            "2023-01-26",
        )
        self.assertHolidayName(f"{name_2012} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_sinai_liberation_day(self):
        name = "عيد تحرير سيناء"
        self.assertNonObservedHolidayName(
            name, (f"{year}-04-25" for year in range(1983, self.end_year))
        )
        self.assertNoHolidayName(name, range(self.start_year, 1983))

        obs_dts = (
            "2021-04-29",
            "2023-04-27",
        )
        self.assertHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_spring_festival(self):
        name = "عيد شم النسيم"
        self.assertHolidayName(
            name,
            "2020-04-20",
            "2021-05-03",
            "2022-04-25",
            "2023-04-17",
            "2024-05-06",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "عيد العمال"
        self.assertNonObservedHolidayName(
            name,
            (
                f"{year}-05-01"
                for year in (*range(self.start_year, 2024), *range(2025, self.end_year))
            ),
        )

        obs_dts = (
            "2022-05-05",
            "2023-05-04",
            "2024-05-05",
        )
        self.assertHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_june_30_revolution_day(self):
        name = "عيد ثورة ٣٠ يونيو"
        self.assertNonObservedHolidayName(
            name, (f"{year}-06-30" for year in range(2014, self.end_year))
        )
        self.assertNoHolidayName(name, range(self.start_year, 2014))

        obs_dts = (
            "2020-07-02",
            "2021-07-01",
            "2023-07-02",
            "2025-07-03",
        )
        self.assertHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_july_23_revolution_day(self):
        name = "عيد ثورة ٢٣ يوليو"
        self.assertNonObservedHolidayName(name, (f"{year}-07-23" for year in self.full_range))

        obs_dts = (
            "2024-07-25",
            "2025-07-24",
        )
        self.assertHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_armed_forces_day(self):
        name = "عيد القوات المسلحة"
        self.assertNonObservedHolidayName(name, (f"{year}-10-06" for year in self.full_range))

        obs_dts = (
            "2020-10-08",
            "2021-10-07",
            "2025-10-09",
        )
        self.assertHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_islamic_new_year(self):
        name = "رأس السنة الهجرية"
        self.assertHolidayName(
            name,
            "2020-08-20",
            "2022-07-30",
            "2025-06-26",
        )
        self.assertIslamicNoEstimatedNonObservedHolidayName(name, self.full_range)

        obs_dts = (
            "2021-08-12",
            "2023-07-20",
            "2024-07-11",
        )
        self.assertHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_prophets_birthday(self):
        name = "المولد النبوي الشريف"
        self.assertHolidayName(
            name,
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-28",
            "2024-09-15",
            "2025-09-04",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_fitr(self):
        name = "عيد الفطر المبارك"
        self.assertHolidayName(
            name,
            "2020-05-24",
            "2020-05-25",
            "2021-05-13",
            "2021-05-14",
            "2022-05-01",
            "2022-05-02",
            "2023-04-21",
            "2023-04-22",
            "2024-04-10",
            "2024-04-11",
            "2025-03-30",
            "2025-03-31",
            "2026-03-20",
            "2026-03-21",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

        obs_dts = (
            "2022-05-04",
            "2023-04-24",
        )
        self.assertHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

        self.assertGovernmentIslamicNoEstimatedHolidayName(
            name,
            "2019-06-03",
            "2019-06-06",
            "2020-05-23",
            "2020-05-26",
            "2021-05-12",
            "2021-05-15",
            "2022-04-30",
            "2022-05-03",
            "2023-04-20",
            "2023-04-23",
            "2024-04-09",
            "2024-04-12",
            "2025-03-29",
            "2025-04-01",
            "2026-03-19",
            "2026-03-22",
        )
        self.assertGovernmentIslamicNoEstimatedHolidayName(name, range(2019, self.end_year))
        self.assertNoGovernmentIslamicNoEstimatedHolidayName(name, range(self.start_year, 2019))

    def test_arafat_day(self):
        name = "وقفة عيد الأضحى المبارك"
        self.assertHolidayName(
            name,
            "2020-07-30",
            "2021-07-19",
            "2022-07-09",
            "2023-06-27",
            "2024-06-15",
            "2025-06-05",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "عيد الأضحى المبارك"
        self.assertHolidayName(
            name,
            "2020-07-31",
            "2020-08-01",
            "2020-08-02",
            "2021-07-17",
            "2021-07-18",
            "2021-07-20",
            "2021-07-21",
            "2021-07-22",
            "2022-07-10",
            "2022-07-11",
            "2022-07-12",
            "2022-07-13",
            "2022-07-14",
            "2023-06-28",
            "2023-06-29",
            "2023-06-30",
            "2024-06-16",
            "2024-06-17",
            "2024-06-18",
            "2025-06-06",
            "2025-06-07",
            "2025-06-08",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

        obs_dts = ("2023-07-03",)
        self.assertHolidayName(f"{name} (يوم تعويضي)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

        self.assertGovernmentIslamicNoEstimatedHolidayName(
            name,
            "2018-08-24",
            "2019-08-14",
            "2020-08-03",
            "2021-07-23",
            "2022-07-13",
            "2023-07-01",
            "2024-06-19",
            "2025-06-09",
        )
        self.assertGovernmentIslamicNoEstimatedHolidayName(name, range(2018, self.end_year))
        self.assertNoGovernmentIslamicNoEstimatedHolidayName(name, range(self.start_year, 2018))

    def test_taba_liberation_day(self):
        name = "عيد تحرير طابا"
        self.assertNoHolidayName(name)
        self.assertSchoolHolidayName(
            name, (f"{year}-03-19" for year in range(2019, self.end_year))
        )
        self.assertNoSchoolHolidayName(name, range(self.start_year, 2019))

    def test_evacuation_day(self):
        name = "عيد الجلاء"
        self.assertNoHolidayName(name)
        self.assertSchoolHolidayName(
            name, (f"{year}-06-18" for year in range(2019, self.end_year))
        )
        self.assertNoSchoolHolidayName(name, range(self.start_year, 2019))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-07", "عيد الميلاد المجيد"),
            ("2024-01-25", "ثورة ٢٥ يناير وعيد الشرطة"),
            ("2024-03-19", "عيد تحرير طابا"),
            ("2024-04-09", "عيد الفطر المبارك"),
            ("2024-04-10", "عيد الفطر المبارك"),
            ("2024-04-11", "عيد الفطر المبارك"),
            ("2024-04-12", "عيد الفطر المبارك"),
            ("2024-04-25", "عيد تحرير سيناء"),
            ("2024-05-05", "عيد العمال (يوم تعويضي)"),
            ("2024-05-06", "عيد شم النسيم"),
            ("2024-06-15", "وقفة عيد الأضحى المبارك"),
            ("2024-06-16", "عيد الأضحى المبارك"),
            ("2024-06-17", "عيد الأضحى المبارك"),
            ("2024-06-18", "عيد الأضحى المبارك; عيد الجلاء"),
            ("2024-06-19", "عيد الأضحى المبارك"),
            ("2024-06-30", "عيد ثورة ٣٠ يونيو"),
            ("2024-07-11", "رأس السنة الهجرية (يوم تعويضي)"),
            ("2024-07-25", "عيد ثورة ٢٣ يوليو (يوم تعويضي)"),
            ("2024-09-15", "المولد النبوي الشريف"),
            ("2024-10-06", "عيد القوات المسلحة"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-07", "Coptic Christmas Day"),
            ("2024-01-25", "January 25th Revolution and National Police Day"),
            ("2024-03-19", "Taba Liberation Day"),
            ("2024-04-09", "Eid al-Fitr"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-04-11", "Eid al-Fitr"),
            ("2024-04-12", "Eid al-Fitr"),
            ("2024-04-25", "Sinai Liberation Day"),
            ("2024-05-05", "Labor Day (observed)"),
            ("2024-05-06", "Spring Festival"),
            ("2024-06-15", "Arafat Day"),
            ("2024-06-16", "Eid al-Adha"),
            ("2024-06-17", "Eid al-Adha"),
            ("2024-06-18", "Eid al-Adha; Evacuation Day"),
            ("2024-06-19", "Eid al-Adha"),
            ("2024-06-30", "June 30 Revolution Day"),
            ("2024-07-11", "Islamic New Year (observed)"),
            ("2024-07-25", "July 23 Revolution Day (observed)"),
            ("2024-09-15", "Prophet's Birthday"),
            ("2024-10-06", "Armed Forces Day"),
        )

    def test_l10n_fr(self):
        self.assertLocalizedHolidays(
            "fr",
            ("2024-01-07", "Fête de Noël"),
            ("2024-01-25", "Révolution de 25 Janvier et la fête de la Police"),
            ("2024-03-19", "Fête de la libération de Taba"),
            ("2024-04-09", "Aïd-Al-Fitr"),
            ("2024-04-10", "Aïd-Al-Fitr"),
            ("2024-04-11", "Aïd-Al-Fitr"),
            ("2024-04-12", "Aïd-Al-Fitr"),
            ("2024-04-25", "Fête de la Libération du Sinaï"),
            ("2024-05-05", "Fête du Travail (observé)"),
            ("2024-05-06", "Cham Al-Nessim"),
            ("2024-06-15", "Le jour d'Arafat"),
            ("2024-06-16", "Aïd Al-Adha"),
            ("2024-06-17", "Aïd Al-Adha"),
            ("2024-06-18", "Aïd Al-Adha; Jour d'évacuation"),
            ("2024-06-19", "Aïd Al-Adha"),
            ("2024-06-30", "Fête de la Révolution du 30 Juin"),
            ("2024-07-11", "Fête de l'Hégire (observé)"),
            ("2024-07-25", "Fête de la Révolution du 23 Juillet (observé)"),
            ("2024-09-15", "Naissance du Prophète"),
            ("2024-10-06", "Fête des Forces Armées"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.djibouti import Djibouti
from tests.common import CommonCountryTests


class TestDjibouti(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Djibouti)

    def test_new_years_day(self):
        self.assertHolidayName("Nouvel an", (f"{year}-01-01" for year in self.full_range))

    def test_labor_day(self):
        self.assertHolidayName("Fête du travail", (f"{year}-05-01" for year in self.full_range))

    def test_independence_day(self):
        self.assertHolidayName(
            "Fête de l'indépendance", (f"{year}-06-27" for year in self.full_range)
        )
        self.assertHolidayName(
            "Fête de l'indépendance deuxième jour", (f"{year}-06-28" for year in self.full_range)
        )

    def test_christmas_day(self):
        self.assertHolidayName("Noël", (f"{year}-12-25" for year in self.full_range))

    def test_isra_and_miraj(self):
        name = "Al Isra et Al Mirague"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-03-22",
            "2021-03-11",
            "2022-02-28",
            "2023-02-18",
            "2024-02-08",
            "2025-01-27",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_fitr(self):
        name = "Eid al-Fitr"
        name_day2 = "Eid al-Fitr deuxième jour"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        self.assertIslamicNoEstimatedHolidayName(name_day2, self.full_range)

    def test_arafat_day(self):
        name = "Arafat"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-07-30",
            "2021-07-19",
            "2022-07-08",
            "2023-06-27",
            "2024-06-15",
            "2025-06-05",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "Eid al-Adha"
        name_day2 = "Eid al-Adha deuxième jour"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        self.assertIslamicNoEstimatedHolidayName(name_day2, self.full_range)

    def test_islamic_new_year(self):
        name = "Nouvel an musulman"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-08-20",
            "2021-08-09",
            "2022-07-30",
            "2023-07-19",
            "2024-07-07",
            "2025-06-26",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_prophet_muhammads_birthday(self):
        name = "Anniversaire du prophète Muhammad"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
            "2025-09-04",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_2019(self):
        self.assertHolidaysInYear(
            2019,
            ("2019-01-01", "Nouvel an"),
            ("2019-04-03", "Al Isra et Al Mirague (estimé)"),
            ("2019-05-01", "Fête du travail"),
            ("2019-06-04", "Eid al-Fitr (estimé)"),
            ("2019-06-05", "Eid al-Fitr deuxième jour (estimé)"),
            ("2019-06-27", "Fête de l'indépendance"),
            ("2019-06-28", "Fête de l'indépendance deuxième jour"),
            ("2019-08-10", "Arafat (estimé)"),
            ("2019-08-11", "Eid al-Adha (estimé)"),
            ("2019-08-12", "Eid al-Adha deuxième jour (estimé)"),
            ("2019-08-31", "Nouvel an musulman (estimé)"),
            ("2019-11-09", "Anniversaire du prophète Muhammad (estimé)"),
            ("2019-12-25", "Noël"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Nouvel an"),
            ("2022-02-28", "Al Isra et Al Mirague (estimé)"),
            ("2022-05-01", "Fête du travail"),
            ("2022-05-02", "Eid al-Fitr (estimé)"),
            ("2022-05-03", "Eid al-Fitr deuxième jour (estimé)"),
            ("2022-06-27", "Fête de l'indépendance"),
            ("2022-06-28", "Fête de l'indépendance deuxième jour"),
            ("2022-07-08", "Arafat (estimé)"),
            ("2022-07-09", "Eid al-Adha (estimé)"),
            ("2022-07-10", "Eid al-Adha deuxième jour (estimé)"),
            ("2022-07-30", "Nouvel an musulman (estimé)"),
            ("2022-10-08", "Anniversaire du prophète Muhammad (estimé)"),
            ("2022-12-25", "Noël"),
        )

    def test_l10n_ar(self):
        self.assertLocalizedHolidays(
            "ar",
            ("2022-01-01", "يوم السنة الجديدة"),
            ("2022-02-28", "الإسراء والمعراج (تقديري)"),
            ("2022-05-01", "عيد العمال"),
            ("2022-05-02", "عيد الفطر (تقديري)"),
            ("2022-05-03", "عطلة عيد الفطر (تقديري)"),
            ("2022-06-27", "عيد الإستقلال"),
            ("2022-06-28", "عطلة عيد الاستقلال"),
            ("2022-07-08", "يوم عرفة (تقديري)"),
            ("2022-07-09", "عيد الأضحى (تقديري)"),
            ("2022-07-10", "عطلة عيد الأضحى (تقديري)"),
            ("2022-07-30", "رأس السنة الهجرية (تقديري)"),
            ("2022-10-08", "عيد المولد النبوي (تقديري)"),
            ("2022-12-25", "عيد الميلاد المجيد"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-02-28", "Isra' and Mi'raj (estimated)"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-02", "Eid al-Fitr (estimated)"),
            ("2022-05-03", "Eid al-Fitr Holiday (estimated)"),
            ("2022-06-27", "Independence Day"),
            ("2022-06-28", "Independence Day Holiday"),
            ("2022-07-08", "Arafat Day (estimated)"),
            ("2022-07-09", "Eid al-Adha (estimated)"),
            ("2022-07-10", "Eid al-Adha Holiday (estimated)"),
            ("2022-07-30", "Islamic New Year (estimated)"),
            ("2022-10-08", "Prophet Muhammad's Birthday (estimated)"),
            ("2022-12-25", "Christmas Day"),
        )



#!/usr/bin/env python3

#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import importlib
import inspect
import re
import sys
from concurrent.futures import ProcessPoolExecutor
from copy import deepcopy
from datetime import datetime
from pathlib import Path
from time import perf_counter

from git import Repo, exc
from lingva.extract import extract as create_pot_file
from lingva.extract import _location_sort_key
from polib import POFile, pofile

sys.path.insert(0, str(Path.cwd()))  # Make holidays visible.
from holidays import __version__ as package_version
from holidays.registry import COUNTRIES, FINANCIAL

WRAP_WIDTH = 99
HEADER_PATH = Path("docs/file_header.txt")
MSGID_BUGS_ADDRESS = "l10n@vacanza.dev"
REQUIRED_METADATA_KEYS = frozenset(
    {
        "Project-Id-Version",
        "Report-Msgid-Bugs-To",
        "POT-Creation-Date",
        "PO-Revision-Date",
        "Last-Translator",
        "Language-Team",
        "Language",
        "MIME-Version",
        "Content-Type",
        "Content-Transfer-Encoding",
        "X-Source-Language",
    }
)

DATE_PATTERN = re.compile(r"\d{4}-\d{2}-\d{2} \d{2}:\d{2}[+-]\d{4}")
TRANSLATOR_PATTERN = re.compile(r"[^\s<]+(?:\s+[^\s<]+)*\s+<[^@\s<]+@[^@\s<.]+(?:\.[^@\s<.]+)+>")


class POGenerator:
    """Generates .po files for supported country/market entities."""

    _author_identity: str = ""
    _license_header: str = ""
    _locale_path: Path = Path("holidays/locale")

    @staticmethod
    def _get_author_identity() -> str:
        result = "FULL NAME <EMAIL@EXAMPLE.COM>"
        try:
            repo = Repo(Path(__file__).parents[2])
            config = repo.config_reader()
            name = config.get_value("user", "name", "")
            email = config.get_value("user", "email", "")
            if name and email and TRANSLATOR_PATTERN.fullmatch(identity := f"{name} <{email}>"):
                result = identity
        except exc.GitError:
            pass

        return result

    @staticmethod
    def _get_license_header() -> str:
        """Reads and formats the license header from docs/file_header.txt."""
        if not HEADER_PATH.exists():
            return ""

        if not (content := HEADER_PATH.read_text(encoding="utf-8").lstrip("\n")):
            return ""

        return (
            "\n".join(
                f"# {stripped}" if (stripped := line.rstrip()) else "#"
                for line in content.splitlines()
            )
            + "\n#"
        )

    @staticmethod
    def _is_po_equal(pofile1: POFile, pofile2: POFile) -> bool:
        """Compare two .po files considering comments."""
        if len(pofile1) != len(pofile2):
            return False

        for entry1, entry2 in zip(pofile1, pofile2):
            if entry1 != entry2:
                return False

            if entry1.comment != entry2.comment or entry1.tcomment != entry2.tcomment:
                return False

        return True

    @classmethod
    def _init_worker(cls, author_identity: str, license_header: str) -> None:
        """Initialize worker with shared data for multiprocessing."""
        cls._author_identity = author_identity
        cls._license_header = license_header

    @classmethod
    def _apply_metadata(
        cls, po_file: POFile, current_lang: str, default_language: str, timestamp: str
    ) -> None:
        """Applies metadata to a .po file."""
        forced_metadata = {
            "Project-Id-Version": f"Holidays {package_version}",
            "Report-Msgid-Bugs-To": MSGID_BUGS_ADDRESS,
            "PO-Revision-Date": timestamp,
            "Last-Translator": cls._author_identity,
            "Language-Team": "Holidays Localization Team",
            "Language": current_lang,
            "MIME-Version": "1.0",
            "Content-Type": "text/plain; charset=UTF-8",
            "Content-Transfer-Encoding": "8bit",
            "X-Source-Language": default_language,
        }
        default_metadata = {
            "POT-Creation-Date": timestamp,
        }
        po_file.metadata = default_metadata | po_file.metadata | forced_metadata

    @classmethod
    def _write_po_header(
        cls, po_path: Path, docstring: str, current_language: str, default_language: str
    ) -> None:
        """Strips gettext boilerplate and prepends the license header + desc line."""
        content = po_path.read_text(encoding="utf-8")
        content = (
            content.split("#, fuzzy", 1)[1].lstrip()
            if content.startswith("# SOME DESCRIPTIVE TITLE")
            else content.lstrip()
        )
        if content.startswith("#  holidays\n#  --------"):
            return

        desc_line = None
        if docstring:
            if display_name := docstring.split("\n", maxsplit=1)[0].removesuffix(" holidays."):
                lang_tag = (
                    f" {current_language} localization"
                    if current_language != default_language
                    else ""
                )
                desc_line = f"# {display_name} holidays{lang_tag}."

        parts = []
        if cls._license_header:
            parts.append(cls._license_header)
        if desc_line and desc_line not in content:
            parts.append(desc_line)
        if parts:
            parts.append("#\n")
            po_path.write_text(  # NOSONAR
                "\n".join(parts) + content, encoding="utf-8", newline="\n"
            )

    @staticmethod
    def _process_entity_worker(args: tuple[str, Path, tuple[str, ...]]) -> tuple[str, POFile]:
        """Process a single entity: create .pot, default .po, and return update tasks."""
        entity_code, class_file_path, supported_languages = args

        pot_path = POGenerator._locale_path / "pot"
        pot_path.mkdir(parents=True, exist_ok=True)
        pot_file_path = pot_path / f"{entity_code}.pot"

        create_pot_file(
            sources=[class_file_path],
            keywords=["tr"],
            output=pot_file_path,
            package_name="Holidays",
            package_version=package_version,
            width=WRAP_WIDTH,
            allow_empty=True,
            msgid_bugs_address=MSGID_BUGS_ADDRESS,
        )

        pot_file = pofile(str(pot_file_path), wrapwidth=WRAP_WIDTH)

        for lang in supported_languages:
            lang_directory = POGenerator._locale_path / lang / "LC_MESSAGES"
            lang_directory.mkdir(parents=True, exist_ok=True)
            lang_po_path = lang_directory / f"{entity_code}.po"

            if not lang_po_path.exists():
                pot_file.save(str(lang_po_path), newline="\n")

        return entity_code, pot_file

    @staticmethod
    def _update_po_file(args: tuple[Path, POFile, str, str]) -> None:
        """Merge .po file with .pot using strict no-change policies."""
        po_path, pot_file, default_language, entity_docstring = args
        po_path = po_path.resolve()
        current_lang = po_path.parent.parent.name

        po_file = pofile(str(po_path), wrapwidth=WRAP_WIDTH)
        po_file_initial = deepcopy(po_file)

        po_file.merge(pot_file)
        po_file.sort(key=_location_sort_key)
        for entry in po_file:
            entry.occurrences.clear()

        metadata = po_file.metadata
        for field_name in ("POT-Creation-Date", "PO-Revision-Date"):
            if not DATE_PATTERN.fullmatch(metadata.get(field_name, "")):
                metadata.pop(field_name, None)
        if not TRANSLATOR_PATTERN.fullmatch(metadata.get("Last-Translator", "")):
            metadata.pop("Last-Translator", None)

        if not POGenerator._is_po_equal(
            po_file, po_file_initial
        ) or not REQUIRED_METADATA_KEYS.issubset(metadata):
            POGenerator._apply_metadata(
                po_file,
                current_lang,
                default_language,
                datetime.now().astimezone().strftime("%Y-%m-%d %H:%M%z"),
            )
            po_file.save(str(po_path), newline="\n")

        POGenerator._write_po_header(po_path, entity_docstring, current_lang, default_language)

    @staticmethod
    def process_entities() -> None:
        """Processes entities in specified directory."""
        entities_data = {}
        entity_worker_data = []

        for entity_type, entity_mapping in (("countries", COUNTRIES), ("financial", FINANCIAL)):
            for path in Path(f"holidays/{entity_type}").glob("*.py"):
                if (mod_name := path.stem) == "__init__":
                    continue

                class_name, entity_code = entity_mapping[mod_name][0:2]
                mod = importlib.import_module(f"holidays.{entity_type}.{mod_name}")

                entity_cls = None
                for cls_name, cls in inspect.getmembers(mod, inspect.isclass):
                    if cls_name == class_name and cls.default_language is not None:
                        entity_cls = cls
                        break

                if not entity_cls:
                    continue

                entity_worker_data.append((entity_code, path, entity_cls.supported_languages))
                entities_data[entity_code] = (
                    entity_cls.default_language,
                    entity_cls.__doc__ or "",
                )

        with ProcessPoolExecutor() as executor:
            entity_pot_mapping = dict(
                executor.map(POGenerator._process_entity_worker, entity_worker_data)
            )

        with ProcessPoolExecutor(
            initializer=POGenerator._init_worker,
            initargs=(POGenerator._get_author_identity(), POGenerator._get_license_header()),
        ) as executor:
            list(
                executor.map(
                    POGenerator._update_po_file,
                    (
                        (path, entity_pot_mapping[path.stem], *entities_data[path.stem])
                        for path in POGenerator._locale_path.rglob("*.po")
                    ),
                )
            )


if __name__ == "__main__":
    po_time_start = perf_counter()
    POGenerator.process_entities()
    po_time_end = perf_counter()
    print(f"[TIMER] Total generate po files runtime: {po_time_end - po_time_start:.2f} seconds")



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import HALF_DAY, SCHOOL
from holidays.countries.bulgaria import Bulgaria
from tests.common import CommonCountryTests, WorkingDayTests


class TestBulgaria(CommonCountryTests, WorkingDayTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Bulgaria, years_non_observed=range(2017, 2026))

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Bulgaria(categories=HALF_DAY, years=range(1996, self.end_year)))
        self.assertNoHolidays(Bulgaria(categories=SCHOOL, years=range(self.start_year, 1992)))

    def test_special_holidays(self):
        self.assertHoliday(
            "2004-04-02",
            "2007-01-02",
            "2025-12-31",
            "2026-01-02",
        )

    def test_substituted_holidays(self):
        self.assertHoliday(
            "2004-05-07",
            "2005-03-04",
            "2005-05-23",
            "2005-09-05",
            "2005-09-23",
            "2006-01-02",
            "2007-04-30",
            "2007-05-25",
            "2007-09-07",
            "2007-12-31",
            "2008-05-02",
            "2008-05-05",
            "2008-12-31",
            "2009-01-02",
            "2009-03-02",
            "2009-05-04",
            "2009-05-05",
            "2009-09-21",
            "2009-12-31",
            "2010-05-07",
            "2010-12-31",
            "2011-03-04",
            "2011-05-23",
            "2011-09-05",
            "2011-09-23",
            "2012-01-02",
            "2012-04-30",
            "2012-05-25",
            "2012-09-07",
            "2012-12-31",
            "2013-05-02",
            "2013-12-23",
            "2013-12-31",
            "2014-05-02",
            "2014-05-05",
            "2014-12-31",
            "2015-01-02",
            "2015-03-02",
            "2015-09-21",
            "2015-12-31",
            "2016-03-04",
            "2016-05-23",
            "2016-09-05",
            "2016-09-23",
        )

    def test_workdays(self):
        self.assertWorkingDay(
            "2004-05-15",
            "2005-03-12",
            "2005-05-28",
            "2005-09-10",
            "2005-09-17",
            "2006-01-28",
            "2007-04-21",
            "2007-06-02",
            "2007-09-15",
            "2007-12-15",
            "2008-05-10",
            "2008-05-17",
            "2008-12-20",
            "2009-01-10",
            "2009-03-14",
            "2009-05-16",
            "2009-05-30",
            "2009-09-26",
            "2009-12-19",
            "2010-05-15",
            "2010-12-11",
            "2011-03-19",
            "2011-05-28",
            "2011-09-03",
            "2011-09-17",
            "2012-01-21",
            "2012-04-21",
            "2012-05-19",
            "2012-09-29",
            "2012-12-15",
            "2013-05-18",
            "2013-12-21",
            "2013-12-14",
            "2014-05-10",
            "2014-05-31",
            "2014-12-13",
            "2015-01-24",
            "2015-03-21",
            "2015-09-12",
            "2015-12-12",
            "2016-03-12",
            "2016-05-14",
            "2016-09-10",
            "2016-09-17",
        )

    def test_new_years_day(self):
        name = "Нова година"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (почивен ден)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_liberation_day(self):
        name = "Ден на Освобождението на България от османско иго"
        self.assertHolidayName(name, (f"{year}-03-03" for year in self.full_range))
        obs_dts = (
            "2018-03-05",
            "2019-03-04",
            "2024-03-04",
        )
        self.assertHolidayName(f"{name} (почивен ден)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Велики петък"
        self.assertHolidayName(
            name,
            "2020-04-17",
            "2021-04-30",
            "2022-04-22",
            "2023-04-14",
            "2024-05-03",
            "2025-04-18",
        )
        self.assertHolidayName(name, range(2010, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2010))

    def test_easter_saturday(self):
        name = "Велика събота"
        self.assertHolidayName(
            name,
            "2020-04-18",
            "2021-05-01",
            "2022-04-23",
            "2023-04-15",
            "2024-05-04",
            "2025-04-19",
        )
        self.assertHolidayName(name, range(2010, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2010))

    def test_easter(self):
        name = "Великден"
        self.assertHolidayName(
            name,
            "2020-04-19",
            "2020-04-20",
            "2021-05-02",
            "2021-05-03",
            "2022-04-24",
            "2022-04-25",
            "2023-04-16",
            "2023-04-17",
            "2024-05-05",
            "2024-05-06",
            "2025-04-20",
            "2025-04-21",
        )
        self.assertHolidayNameCount(name, 2, self.full_range)

    def test_labor_day(self):
        name = "Ден на труда и на международната работническа солидарност"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2021-05-04",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (почивен ден)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_saint_georges_day(self):
        name = "Гергьовден, Ден на храбростта и Българската армия"
        self.assertHolidayName(name, (f"{year}-05-06" for year in range(1999, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1999))
        obs_dts = (
            "2017-05-08",
            "2018-05-07",
            "2023-05-08",
        )
        self.assertHolidayName(f"{name} (почивен ден)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_bulgarian_culture_day(self):
        name_1991 = "Ден на българската просвета и култура и на славянската писменост"
        name_2021 = (
            "Ден на светите братя Кирил и Методий, на българската азбука, "
            "просвета и култура и на славянската книжовност"
        )
        self.assertHolidayName(
            name_1991, (f"{year}-05-24" for year in range(self.start_year, 2021))
        )
        self.assertHolidayName(name_2021, (f"{year}-05-24" for year in range(2021, self.end_year)))
        self.assertNoHolidayName(name_1991, range(2021, self.end_year))
        self.assertNoHolidayName(name_2021, range(self.start_year, 2021))
        obs_dts = ("2025-05-26",)
        self.assertHolidayName(f"{name_1991} (почивен ден)", "2020-05-25")
        self.assertHolidayName(f"{name_2021} (почивен ден)", obs_dts)
        self.assertNoNonObservedHoliday("2020-05-25", obs_dts)

    def test_unification_day(self):
        name = "Ден на Съединението"
        self.assertHolidayName(name, (f"{year}-09-06" for year in range(1998, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1998))
        obs_dts = (
            "2020-09-07",
            "2025-09-08",
        )
        self.assertHolidayName(f"{name} (почивен ден)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_freedom_day(self):
        name = "Ден на свободата"
        self.assertHolidayName(name, "1991-09-09")
        self.assertNoHolidayName(name, range(1992, self.end_year))

    def test_independence_day(self):
        name = "Ден на Независимостта на България"
        self.assertHolidayName(name, (f"{year}-09-22" for year in range(1998, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1998))
        obs_dts = (
            "2018-09-24",
            "2019-09-23",
            "2024-09-23",
        )
        self.assertHolidayName(f"{name} (почивен ден)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_awakening_day(self):
        name = "Ден на народните будители"
        self.assertNoHolidayName(name)
        self.assertSchoolHolidayName(
            name, (f"{year}-11-01" for year in range(1992, self.end_year))
        )
        self.assertNoSchoolHolidayName(name, 1991)

    def test_christmas_eve(self):
        name = "Бъдни вечер"
        self.assertHolidayName(name, (f"{year}-12-24" for year in range(1996, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1996))
        obs_dts = (
            "2017-12-27",
            "2022-12-27",
            "2023-12-27",
        )
        self.assertHolidayName(f"{name} (почивен ден)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

        half_day_name = f"{name} (след 14 ч.)"
        self.assertHalfDayHolidayName(
            half_day_name, (f"{year}-12-24" for year in range(self.start_year, 1996))
        )
        self.assertNoHalfDayHolidayName(half_day_name, range(1996, self.end_year))

    def test_christmas_day(self):
        name = "Рождество Христово"
        self.assertHolidayName(
            name,
            (f"{year}-12-25" for year in self.full_range),
            (f"{year}-12-26" for year in self.full_range),
        )
        obs_dts = (
            "2020-12-28",
            "2021-12-27",
            "2021-12-28",
            "2022-12-28",
        )
        self.assertHolidayName(f"{name} (почивен ден)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Нова година"),
            ("2022-01-03", "Нова година (почивен ден)"),
            ("2022-03-03", "Ден на Освобождението на България от османско иго"),
            ("2022-04-22", "Велики петък"),
            ("2022-04-23", "Велика събота"),
            ("2022-04-24", "Великден"),
            ("2022-04-25", "Великден"),
            ("2022-05-01", "Ден на труда и на международната работническа солидарност"),
            (
                "2022-05-02",
                "Ден на труда и на международната работническа солидарност (почивен ден)",
            ),
            ("2022-05-06", "Гергьовден, Ден на храбростта и Българската армия"),
            (
                "2022-05-24",
                "Ден на светите братя Кирил и Методий, на българската азбука, "
                "просвета и култура и на славянската книжовност",
            ),
            ("2022-09-06", "Ден на Съединението"),
            ("2022-09-22", "Ден на Независимостта на България"),
            ("2022-11-01", "Ден на народните будители"),
            ("2022-12-24", "Бъдни вечер"),
            ("2022-12-25", "Рождество Христово"),
            ("2022-12-26", "Рождество Христово"),
            ("2022-12-27", "Бъдни вечер (почивен ден)"),
            ("2022-12-28", "Рождество Христово (почивен ден)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-03-03", "Liberation Day"),
            ("2022-04-22", "Good Friday"),
            ("2022-04-23", "Holy Saturday"),
            ("2022-04-24", "Easter"),
            ("2022-04-25", "Easter"),
            ("2022-05-01", "Labor Day and International Workers' Solidarity Day"),
            ("2022-05-02", "Labor Day and International Workers' Solidarity Day (observed)"),
            ("2022-05-06", "Saint George's Day, Day of the Bulgarian Army"),
            ("2022-05-24", "Day of Slavonic Alphabet, Bulgarian Enlightenment and Culture"),
            ("2022-09-06", "Unification Day"),
            ("2022-09-22", "Independence Day"),
            ("2022-11-01", "The Day of the People's Awakeners"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Christmas Day"),
            ("2022-12-27", "Christmas Eve (observed)"),
            ("2022-12-28", "Christmas Day (observed)"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-03", "Новий рік (вихідний)"),
            ("2022-03-03", "День визволення Болгарії від османського іга"),
            ("2022-04-22", "Страсна пʼятниця"),
            ("2022-04-23", "Велика субота"),
            ("2022-04-24", "Великдень"),
            ("2022-04-25", "Великдень"),
            ("2022-05-01", "День праці та міжнародної солідарності трудящих"),
            ("2022-05-02", "День праці та міжнародної солідарності трудящих (вихідний)"),
            ("2022-05-06", "День Святого Георгія та День хоробрості і болгарської армії"),
            (
                "2022-05-24",
                "День святих братів Кирила і Мефодія, болгарської писемності, "
                "освіти і культури та словʼянської літератури",
            ),
            ("2022-09-06", "День обʼєднання"),
            ("2022-09-22", "День незалежності Болгарії"),
            ("2022-11-01", "День національних будителів"),
            ("2022-12-24", "Святий вечір"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Різдво Христове"),
            ("2022-12-27", "Святий вечір (вихідний)"),
            ("2022-12-28", "Різдво Христове (вихідний)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import warnings
from unittest import TestCase

from holidays.constants import OPTIONAL, PUBLIC
from holidays.countries.portugal import Portugal
from tests.common import CommonCountryTests


class TestPortugal(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Portugal, years_all_subdivs=range(1910, 2050))

    def setUp(self):
        super().setUp()
        warnings.simplefilter("ignore", category=DeprecationWarning)

    def test_subdiv_deprecation(self):
        self.assertDeprecatedSubdivisions("This subdivision is deprecated and will be removed")

    def test_2014(self):
        # http://www.officeholidays.com/countries/portugal/2014.php
        self.assertHolidaysInYear(
            2014,
            ("2014-01-01", "Ano Novo"),
            ("2014-04-18", "Sexta-feira Santa"),
            ("2014-04-20", "Páscoa"),
            ("2014-04-25", "Dia da Liberdade"),
            ("2014-05-01", "Dia do Trabalhador"),
            ("2014-06-10", "Dia de Portugal, de Camões e das Comunidades Portuguesas"),
            ("2014-08-15", "Assunção de Nossa Senhora"),
            ("2014-12-08", "Imaculada Conceição"),
            ("2014-12-25", "Dia de Natal"),
        )

    def test_2017(self):
        # http://www.officeholidays.com/countries/portugal/2017.php
        self.assertHolidaysInYear(
            2017,
            ("2017-01-01", "Ano Novo"),
            ("2017-04-14", "Sexta-feira Santa"),
            ("2017-04-16", "Páscoa"),
            ("2017-04-25", "Dia da Liberdade"),
            ("2017-05-01", "Dia do Trabalhador"),
            ("2017-06-10", "Dia de Portugal, de Camões e das Comunidades Portuguesas"),
            ("2017-06-15", "Corpo de Deus"),
            ("2017-08-15", "Assunção de Nossa Senhora"),
            ("2017-10-05", "Implantação da República"),
            ("2017-11-01", "Dia de Todos os Santos"),
            ("2017-12-01", "Restauração da Independência"),
            ("2017-12-08", "Imaculada Conceição"),
            ("2017-12-25", "Dia de Natal"),
        )

    def test_district_specific_days(self):
        # Conselho Holidays only starts in 1911
        self.assertNoSubdiv01Holiday("1910-05-12")
        self.assertNoSubdiv02Holiday("1910-05-05")
        self.assertNoSubdiv03Holiday("1910-06-24")
        self.assertNoSubdiv04Holiday("1910-08-22")
        self.assertNoSubdiv05Holiday("1910-04-12")
        self.assertNoSubdiv06Holiday("1910-07-04")
        self.assertNoSubdiv07Holiday("1910-06-29")
        self.assertNoSubdiv08Holiday("1910-09-07")
        self.assertNoSubdiv09Holiday("1910-11-27")
        self.assertNoSubdiv10Holiday("1910-05-22")
        self.assertNoSubdiv11Holiday("1910-06-13")
        self.assertNoSubdiv12Holiday("1910-05-23")
        self.assertNoSubdiv13Holiday("1910-06-24")
        self.assertNoSubdiv14Holiday("1910-03-19")
        self.assertNoSubdiv15Holiday("1910-09-15")
        self.assertNoSubdiv16Holiday("1910-08-20")
        self.assertNoSubdiv17Holiday("1910-06-13")
        self.assertNoSubdiv18Holiday("1910-09-21")
        self.assertNoSubdiv20Holiday("1910-05-16")
        self.assertNoSubdiv30Holiday("1910-07-01", "1910-12-26")

        # 2017 Cases
        self.assertSubdiv01Holiday("2017-05-12")
        self.assertSubdiv02Holiday("2017-05-25")
        self.assertSubdiv03Holiday("2017-06-24")
        self.assertSubdiv04Holiday("2017-08-22")
        self.assertSubdiv05Holiday("2017-05-02")
        self.assertSubdiv06Holiday("2017-07-04")
        self.assertSubdiv07Holiday("2017-06-29")
        self.assertSubdiv08Holiday("2017-09-07")
        self.assertSubdiv09Holiday("2017-11-27")
        self.assertSubdiv10Holiday("2017-05-22")
        self.assertSubdiv11Holiday("2017-06-13")
        self.assertSubdiv12Holiday("2017-05-23")
        self.assertSubdiv13Holiday("2017-06-24")
        self.assertSubdiv14Holiday("2017-03-19")
        self.assertSubdiv15Holiday("2017-09-15")
        self.assertSubdiv16Holiday("2017-08-20")
        self.assertSubdiv17Holiday("2017-06-13")
        self.assertSubdiv18Holiday("2017-09-21")
        self.assertSubdiv20Holiday("2017-06-05")
        self.assertSubdiv30Holiday("2017-07-01", "2017-12-26")

    def test_azores_day(self):
        name = "Dia da Região Autónoma dos Açores"
        self.assertSubdiv20HolidayName(
            name,
            "2016-05-16",
            "2017-06-05",
            "2018-05-21",
            "2019-06-10",
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
        )
        self.assertSubdiv20HolidayName(name, range(1981, self.end_year))
        self.assertNoSubdiv20HolidayName(name, range(1910, 1981))
        self.assertNoHolidayName(name)

    def test_madeira_day(self):
        name_1979 = "Dia da Região Autónoma da Madeira"
        name_1989 = "Dia da Região Autónoma da Madeira e das Comunidades Madeirenses"
        self.assertSubdiv30HolidayName(name_1979, (f"{year}-07-01" for year in range(1979, 1989)))
        self.assertSubdiv30HolidayName(
            name_1989, (f"{year}-07-01" for year in range(1989, self.end_year))
        )
        self.assertNoSubdiv30HolidayName(name_1979, range(1910, 1979), range(1989, self.end_year))
        self.assertNoSubdiv30HolidayName(name_1989, range(1910, 1989))
        self.assertNoHolidayName(name_1979)
        self.assertNoHolidayName(name_1989)

    def test_primeira_oitava(self):
        name = "Primeira Oitava"
        self.assertSubdiv30HolidayName(
            name, (f"{year}-12-26" for year in range(2002, self.end_year))
        )
        self.assertNoSubdiv30HolidayName(name, range(self.start_year, 2002))
        self.assertNoHolidayName(name)

    def test_optional_holidays(self):
        self.assertOptionalHoliday(
            "2017-02-28",
            "2017-06-13",
            "2017-12-24",
            "2017-12-26",
            "2017-12-31",
            "2018-02-13",
            "2018-06-13",
            "2018-12-24",
            "2018-12-26",
            "2018-12-31",
            "2019-03-05",
            "2019-06-13",
            "2019-12-24",
            "2019-12-26",
            "2019-12-31",
        )

    def test_deprecated(self):
        self.assertEqual(
            Portugal(subdiv="Ext", years=2022).keys(),
            Portugal(categories=(OPTIONAL, PUBLIC), years=2022).keys(),
        )

    def test_corpus_christi(self):
        name = "Corpo de Deus"
        self.assertHolidayName(
            name,
            "2016-05-26",
            "2017-06-15",
            "2018-05-31",
            "2019-06-20",
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
        )
        self.assertHolidayName(name, range(self.start_year, 2013), range(2016, self.end_year))
        self.assertNoHolidayName(name, range(2013, 2016))

    def test_republic_day(self):
        name = "Implantação da República"
        self.assertHolidayName(
            name, (f"{year}-10-05" for year in (*range(1910, 2013), *range(2016, self.end_year)))
        )
        self.assertNoHolidayName(name, range(self.start_year, 1910), range(2013, 2016))

    def test_all_saints_day(self):
        name = "Dia de Todos os Santos"
        self.assertHolidayName(
            name,
            (
                f"{year}-11-01"
                for year in (*range(self.start_year, 2013), *range(2016, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, range(2013, 2016))

    def test_restoration_of_independence_day(self):
        name = "Restauração da Independência"
        self.assertHolidayName(
            name, (f"{year}-12-01" for year in (*range(1823, 2013), *range(2016, self.end_year)))
        )
        self.assertNoHolidayName(name, range(self.start_year, 1823), range(2013, 2016))

    def test_freedom_day(self):
        name = "Dia da Liberdade"
        self.assertHolidayName(name, (f"{year}-04-25" for year in range(1974, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1974))

    def test_labor_day(self):
        name = "Dia do Trabalhador"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1974, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1974))

    def test_portugal_day(self):
        name_1911 = "Dia de Portugal"
        name_1933 = "Dia de Camões, de Portugal e da Raça"
        name_1978 = "Dia de Portugal, de Camões e das Comunidades Portuguesas"
        self.assertHolidayName(
            name_1911, (f"{year}-06-10" for year in (*range(1911, 1933), *range(1974, 1978)))
        )
        self.assertHolidayName(name_1933, (f"{year}-06-10" for year in range(1933, 1974)))
        self.assertHolidayName(name_1978, (f"{year}-06-10" for year in range(1978, self.end_year)))
        self.assertNoHolidayName(
            name_1911, range(self.start_year, 1911), range(1933, 1974), range(1978, self.end_year)
        )
        self.assertNoHolidayName(
            name_1933, range(self.start_year, 1933), range(1974, self.end_year)
        )
        self.assertNoHolidayName(name_1978, range(self.start_year, 1978))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2018-01-01", "Ano Novo"),
            ("2018-02-13", "Carnaval"),
            ("2018-03-19", "Dia de São José"),
            ("2018-03-30", "Sexta-feira Santa"),
            ("2018-04-01", "Páscoa"),
            ("2018-04-17", "Dia de Nossa Senhora de Mércoles"),
            ("2018-04-25", "Dia da Liberdade"),
            ("2018-05-01", "Dia do Trabalhador"),
            ("2018-05-10", "Quinta-feira da Ascensão"),
            ("2018-05-12", "Dia de Santa Joana"),
            ("2018-05-21", "Dia da Região Autónoma dos Açores"),
            ("2018-05-22", "Dia do Município de Leiria"),
            ("2018-05-23", "Dia do Município de Portalegre"),
            ("2018-05-31", "Corpo de Deus"),
            ("2018-06-10", "Dia de Portugal, de Camões e das Comunidades Portuguesas"),
            ("2018-06-13", "Dia de Santo António"),
            ("2018-06-24", "Dia de São João"),
            ("2018-06-29", "Dia de São Pedro"),
            ("2018-07-01", "Dia da Região Autónoma da Madeira e das Comunidades Madeirenses"),
            ("2018-07-04", "Dia de Santa Isabel"),
            ("2018-08-15", "Assunção de Nossa Senhora"),
            ("2018-08-20", "Dia de Nossa Senhora da Agonia"),
            ("2018-08-22", "Dia de Nossa Senhora das Graças"),
            ("2018-09-07", "Dia do Município de Faro"),
            ("2018-09-15", "Dia de Bocage"),
            ("2018-09-21", "Dia de São Mateus"),
            ("2018-10-05", "Implantação da República"),
            ("2018-11-01", "Dia de Todos os Santos"),
            ("2018-11-27", "Dia do Município da Guarda"),
            ("2018-12-01", "Restauração da Independência"),
            ("2018-12-08", "Imaculada Conceição"),
            ("2018-12-24", "Véspera de Natal"),
            ("2018-12-25", "Dia de Natal"),
            ("2018-12-26", "26 de Dezembro; Primeira Oitava"),
            ("2018-12-31", "Véspera de Ano Novo"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2018-01-01", "New Year's Day"),
            ("2018-02-13", "Carnival"),
            ("2018-03-19", "Saint Joseph's Day"),
            ("2018-03-30", "Good Friday"),
            ("2018-04-01", "Easter Sunday"),
            ("2018-04-17", "Feast of Our Lady of Mércoles"),
            ("2018-04-25", "Freedom Day"),
            ("2018-05-01", "Labor Day"),
            ("2018-05-10", "Ascension Day"),
            ("2018-05-12", "Saint Joanna's Day"),
            ("2018-05-21", "Day of the Autonomous Region of the Azores"),
            ("2018-05-22", "Municipal Holiday of Leiria"),
            ("2018-05-23", "Municipal Holiday of Portalegre"),
            ("2018-05-31", "Corpus Christi"),
            ("2018-06-10", "Day of Portugal, Camões, and the Portuguese Communities"),
            ("2018-06-13", "Saint Anthony's Day"),
            ("2018-06-24", "Saint John's Day"),
            ("2018-06-29", "Saint Peter's Day"),
            ("2018-07-01", "Day of the Autonomous Region of Madeira and the Madeiran Communities"),
            ("2018-07-04", "Saint Elizabeth's Day"),
            ("2018-08-15", "Assumption Day"),
            ("2018-08-20", "Feast of Our Lady of Sorrows"),
            ("2018-08-22", "Feast of Our Lady of Graces"),
            ("2018-09-07", "Municipal Holiday of Faro"),
            ("2018-09-15", "Bocage Day"),
            ("2018-09-21", "Saint Matthew's Day"),
            ("2018-10-05", "Republic Day"),
            ("2018-11-01", "All Saints' Day"),
            ("2018-11-27", "Municipal Holiday of Guarda"),
            ("2018-12-01", "Restoration of Independence Day"),
            ("2018-12-08", "Immaculate Conception"),
            ("2018-12-24", "Christmas Eve"),
            ("2018-12-25", "Christmas Day"),
            ("2018-12-26", "1st Octave; Second Day of Christmas"),
            ("2018-12-31", "New Year's Eve"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2018-01-01", "Новий рік"),
            ("2018-02-13", "Карнавал"),
            ("2018-03-19", "День Святого Йосипа"),
            ("2018-03-30", "Страсна пʼятниця"),
            ("2018-04-01", "Великдень"),
            ("2018-04-17", "День Богоматері Меркольської"),
            ("2018-04-25", "День Свободи"),
            ("2018-05-01", "День праці"),
            ("2018-05-10", "Вознесіння Господнє"),
            ("2018-05-12", "День Святої Йоанни"),
            ("2018-05-21", "День автономного регіону Азорських островів"),
            ("2018-05-22", "День муніципалітету Лейрія"),
            ("2018-05-23", "День муніципалітету Порталегре"),
            ("2018-05-31", "Свято Тіла і Крові Христових"),
            ("2018-06-10", "День Португалії, Камоенса і португальських громад"),
            ("2018-06-13", "День Святого Антонія"),
            ("2018-06-24", "День Святого Івана"),
            ("2018-06-29", "День Святого Петра"),
            ("2018-07-01", "День автономного регіону Мадейра та мадейрських громад"),
            ("2018-07-04", "День Святої Єлизавети"),
            ("2018-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2018-08-20", "День Богоматері Страждання"),
            ("2018-08-22", "День Богоматері Милосердя"),
            ("2018-09-07", "День муніципалітету Фару"),
            ("2018-09-15", "День Бокажі"),
            ("2018-09-21", "День Святого Матвія"),
            ("2018-10-05", "День Республіки"),
            ("2018-11-01", "День усіх святих"),
            ("2018-11-27", "День муніципалітету Гуарда"),
            ("2018-12-01", "День відновлення незалежності"),
            ("2018-12-08", "Непорочне зачаття Діви Марії"),
            ("2018-12-24", "Святий вечір"),
            ("2018-12-25", "Різдво Христове"),
            ("2018-12-26", "Другий день Різдва; Перша октава"),
            ("2018-12-31", "Переддень Нового року"),
        )



#!/usr/bin/env python3


#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import argparse
import json
import shutil
import sys
import warnings
from concurrent.futures import ProcessPoolExecutor
from pathlib import Path
from time import perf_counter

sys.path.insert(0, str(Path.cwd()))  # Make holidays visible.
from holidays import (
    country_holidays,
    financial_holidays,
    list_supported_countries,
    list_supported_financial,
)


class SnapshotGenerator:
    """Creates a snapshot of available holidays for supported entities."""

    years = range(1950, 2051)

    def __init__(self) -> None:
        arg_parser = argparse.ArgumentParser()
        entities_group = arg_parser.add_mutually_exclusive_group()
        entities_group.add_argument(
            "-c",
            "--country",
            action="extend",
            nargs="+",
            default=[],
            help="Country codes to use for snapshot generation",
            type=str,
        )
        entities_group.add_argument(
            "-m",
            "--market",
            action="extend",
            nargs="+",
            default=[],
            help="Market codes to use for snapshot generation",
            type=str,
        )
        self.args = arg_parser.parse_args()

    @staticmethod
    def prepare_snapshot_directory(snapshot_path: Path) -> None:
        """Prepare a directory for snapshots."""
        shutil.rmtree(snapshot_path, ignore_errors=True)
        snapshot_path.mkdir(parents=True, exist_ok=True)

    @staticmethod
    def save(snapshot: dict, file_path: Path) -> None:
        """Save snapshot to a JSON file."""
        file_path.write_text(
            json.dumps(
                {str(dt): name for dt, name in sorted(snapshot.items())},
                ensure_ascii=False,
                indent=4,
            )
            + "\n",  # Get along with pre-commit.
            encoding="utf-8",
            newline="\n",
        )

    @staticmethod
    def _country_subdiv_snapshot_worker(
        args: tuple[str, str | None, tuple[str, ...], range, Path],
    ) -> None:
        """Worker for generating country holiday snapshots."""
        warnings.simplefilter("ignore")
        country_code, subdiv, supported_categories, years, snapshot_path = args
        filename = f"{country_code}_{(subdiv or 'COMMON').replace(' ', '_').upper()}.json"
        file_path = snapshot_path / filename
        snapshot = country_holidays(
            country_code,
            subdiv=subdiv,
            years=years,
            categories=supported_categories,
            language="en_US",
        )
        SnapshotGenerator.save(snapshot, file_path)

    @staticmethod
    def _financial_snapshot_worker(args: tuple[str, tuple[str, ...], range, Path]) -> None:
        """Worker for generating financial market holiday snapshots."""
        warnings.simplefilter("ignore")
        market_code, supported_categories, years, snapshot_path = args
        file_path = snapshot_path / f"{market_code}.json"
        snapshot = financial_holidays(
            market_code,
            years=years,
            categories=supported_categories,
            language="en_US",
        )
        SnapshotGenerator.save(snapshot, file_path)

    def generate_country_snapshots(self) -> None:
        """Generates country snapshots."""
        if self.args.market:
            return None

        supported_countries = list_supported_countries(include_aliases=False)
        country_list = self.args.country or list(supported_countries.keys())
        if unknown_countries := set(country_list).difference(supported_countries.keys()):
            raise ValueError(f"Countries {', '.join(unknown_countries)} not available")

        snapshot_path = Path("snapshots/countries")
        if not self.args.country:
            self.prepare_snapshot_directory(snapshot_path)

        work_items: list[tuple[str, str | None, tuple[str, ...], range, Path]] = []
        for country_code in country_list:
            country = country_holidays(country_code)
            for subdiv in (None, *country.subdivisions):
                work_items.append(
                    (country_code, subdiv, country.supported_categories, self.years, snapshot_path)
                )
        with ProcessPoolExecutor() as executor:
            list(executor.map(SnapshotGenerator._country_subdiv_snapshot_worker, work_items))

    def generate_financial_snapshots(self) -> None:
        """Generates financial snapshots."""
        if self.args.country:
            return None

        supported_markets = list_supported_financial(include_aliases=False)
        market_list = self.args.market or list(supported_markets.keys())
        if unknown_markets := set(market_list).difference(supported_markets.keys()):
            raise ValueError(f"Markets {', '.join(unknown_markets)} not available")

        snapshot_path = Path("snapshots/financial")
        if not self.args.market:
            self.prepare_snapshot_directory(snapshot_path)

        work_items: list[tuple[str, tuple[str, ...], range, Path]] = []
        for market_code in market_list:
            market = financial_holidays(market_code)
            work_items.append(
                (market_code, market.supported_categories, self.years, snapshot_path)
            )
        with ProcessPoolExecutor() as executor:
            list(executor.map(SnapshotGenerator._financial_snapshot_worker, work_items))

    def run(self):
        """Runs snapshot files generation process."""
        self.generate_country_snapshots()
        self.generate_financial_snapshots()


if __name__ == "__main__":
    total_time_start = perf_counter()
    SnapshotGenerator().run()
    total_time_end = perf_counter()
    print(f"[TIMER] Total snapshot runtime: {total_time_end - total_time_start:.2f} seconds")



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.chad import Chad
from tests.common import CommonCountryTests


class TestChad(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Chad)

    def test_special_holidays(self):
        self.assertHoliday("2021-04-23")

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_international_womens_day(self):
        name = "International Women's Day"
        self.assertHolidayName(name, (f"{year}-03-08" for year in self.full_range))
        obs_dts = (
            "2015-03-09",
            "2020-03-09",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "Labour Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2011-05-02",
            "2016-05-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-08-11" for year in self.full_range))
        obs_dts = (
            "2013-08-12",
            "2019-08-12",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_all_saints_day(self):
        self.assertHolidayName("All Saints' Day", (f"{year}-11-01" for year in self.full_range))

    def test_republic_day(self):
        name = "Republic Day"
        self.assertHolidayName(name, (f"{year}-11-28" for year in self.full_range))
        obs_dts = (
            "2010-11-29",
            "2021-11-29",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_freedom_and_democracy_day(self):
        name = "Freedom and Democracy Day"
        self.assertHolidayName(name, (f"{year}-12-01" for year in range(1991, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1991))
        obs_dts = (
            "2013-12-02",
            "2019-12-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in self.full_range))

    def test_eid_al_fitr(self):
        name = "Eid al-Fitr"
        self.assertHolidayName(
            name,
            "2018-06-15",
            "2019-06-04",
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "Eid al-Adha"
        self.assertHolidayName(
            name,
            "2018-08-22",
            "2019-08-11",
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_prophets_birthday(self):
        name = "Mawlid"
        self.assertHolidayName(
            name,
            "2018-11-21",
            "2019-11-09",
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-03-08", "International Women's Day"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labour Day"),
            ("2022-05-02", "Eid al-Fitr; Labour Day (observed)"),
            ("2022-07-09", "Eid al-Adha"),
            ("2022-08-11", "Independence Day"),
            ("2022-10-08", "Mawlid"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-11-28", "Republic Day"),
            ("2022-12-01", "Freedom and Democracy Day"),
            ("2022-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.lesotho import Lesotho
from tests.common import CommonCountryTests


class TestLesotho(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Lesotho)

    def test_special_holidays(self):
        self.assertHoliday("2002-05-25")

    def test_new_years_day(self):
        self.assertHolidayName("New Year's Day", (f"{year}-01-01" for year in self.full_range))

    def test_moshoeshoes_day(self):
        self.assertHolidayName("Moshoeshoe's Day", (f"{year}-03-11" for year in self.full_range))

    def test_heroes_day(self):
        name = "Heroes Day"
        self.assertHolidayName(name, (f"{year}-04-04" for year in range(self.start_year, 2003)))
        self.assertNoHolidayName(name, range(2003, self.end_year))

    def test_africa_heroes_day(self):
        name = "Africa/Heroes Day"
        self.assertHolidayName(name, (f"{year}-05-25" for year in range(2003, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2003))

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_workers_day(self):
        self.assertHolidayName("Workers' Day", (f"{year}-05-01" for year in self.full_range))

    def test_ascension_day(self):
        name = "Ascension Day"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_kings_birthday(self):
        name = "King's Birthday"
        self.assertHolidayName(
            name,
            (f"{year}-05-02" for year in range(self.start_year, 1998)),
            (f"{year}-07-17" for year in range(1998, self.end_year)),
        )

    def test_independence_day(self):
        self.assertHolidayName("Independence Day", (f"{year}-10-04" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in self.full_range))

    def test_boxing_day(self):
        self.assertHolidayName("Boxing Day", (f"{year}-12-26" for year in self.full_range))

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-03-11", "Moshoeshoe's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Workers' Day"),
            ("2022-05-25", "Africa/Heroes Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-07-17", "King's Birthday"),
            ("2022-10-04", "Independence Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SAT_TO_PREV_FRI, SUN_TO_NEXT_MON


class SaoTomeAndPrincipe(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Sao Tome and Principe holidays.

    References:
        * <https://web.archive.org/web/20190319201800/http://www.mnec.gov.st/index.php/o-pais?showall=1>
        * <https://web.archive.org/web/20241102204908/https://visitsaotomeprincipe.st/pt/informacoes-uteis/feriados>
        * <https://en.wikipedia.org/wiki/Public_holidays_in_São_Tomé_and_Príncipe>
        * <https://web.archive.org/web/20250320223941/https://www.timeanddate.com/holidays/sao-tome-and-principe/>
    """

    country = "ST"
    default_language = "pt_ST"
    # %s (observed).
    observed_label = tr("%s (observado)")
    start_year = 2014
    subdivisions = (
        "01",  # Água Grande.
        "02",  # Cantagalo.
        "03",  # Caué.
        "04",  # Lembá.
        "05",  # Lobata.
        "06",  # Mé-Zóchi.
        "P",  # Príncipe.
    )
    subdivisions_aliases = {
        # Districts.
        "Água Grande": "01",
        "Cantagalo": "02",
        "Caué": "03",
        "Lembá": "04",
        "Lobata": "05",
        "Mé-Zóchi": "06",
        # Autonomous Region.
        "Príncipe": "P",
    }
    supported_languages = ("en_US", "pt_ST")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON + SAT_TO_PREV_FRI)
        # Holidays are observed on the next Monday if on Sunday, or previous Friday if on Saturday.
        # Based on common government practices since at least 2020.
        kwargs.setdefault("observed_since", 2020)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        name = tr("Ano Novo")
        self._add_observed(self._add_new_years_day(name))
        self._add_observed(self._next_year_new_years_day, name=name, rule=SAT_TO_PREV_FRI)

        # Day of King Amador.
        self._add_observed(self._add_holiday_jan_4(tr("Dia do Rei Amador")))

        # Martyrs' Day.
        self._add_observed(self._add_holiday_feb_3(tr("Dia dos Mártires")))

        # Worker's Day.
        self._add_observed(self._add_labor_day(tr("Dia do Trabalhador")))

        # Independence Day.
        self._add_observed(self._add_holiday_jul_12(tr("Dia da Independência")))

        # Armed Forces Day.
        self._add_observed(self._add_holiday_sep_6(tr("Dia das Forças Armadas")))

        # Agricultural Reform Day.
        self._add_observed(self._add_holiday_sep_30(tr("Dia da Reforma Agrária")))

        if self._year >= 2019:
            # São Tomé Day.
            self._add_observed(self._add_holiday_dec_21(tr("Dia de São Tomé")))

        # Christmas Day.
        self._add_observed(self._add_christmas_day(tr("Natal")))

    def _populate_subdiv_p_public_holidays(self):
        # Discovery of Príncipe Island.
        self._add_observed(self._add_holiday_jan_17(tr("Descobrimento da Ilha do Príncipe")))

        # Autonomy Day.
        self._add_observed(self._add_holiday_apr_29(tr("Dia da Autonomia do Príncipe")))

        # São Lourenço Day.
        self._add_observed(self._add_holiday_aug_15(tr("Dia de São Lourenço")))


class ST(SaoTomeAndPrincipe):
    pass


class STP(SaoTomeAndPrincipe):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_TO_NEXT_TUE,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
)


class CookIslands(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Cook Islands holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_the_Cook_Islands>
        * [Public Holidays Amendment Act 1970-71](https://web.archive.org/web/20250530101809/https://cookislandslaws.gov.ck/#/Laws-as-Made?id=414&name=Public%20Holidays%20Amendment%20Act%201970-71%20No%2029)
        * [Public Holidays Act 1999 (Law as Made)](https://web.archive.org/web/20250530120928/https://cookislandslaws.gov.ck/#/Laws-as-Made?id=1241&name=Public%20Holidays%20Act%201999%20No%2017)
        * [Public Holidays (Aitutaki Gospel Day) Order 1989](https://web.archive.org/web/20250530121249/https://parliament.gov.ck/wp-content/uploads/2022/07/Public-Holidays-Aitutaki-Gospel-Day-Order-1990-No.-9.pdf)
        * [Public Holidays Act 1999 (Consolidated Law)](https://web.archive.org/web/20250530101809/https://cookislandslaws.gov.ck/#/InternalConsolidatedLaws?legalId=LOCI.PHA99&parent_id=Public%20Holidays%20Act%201999)
        * [Gospel Days](https://web.archive.org/web/20110928152006/http://www.cookislands.org.uk/gospelday.html)
        * [Public Holidays (Ra o te Ui Ariki) Amendment Act 2011](https://web.archive.org/web/20250530101809/https://cookislandslaws.gov.ck/#/Laws-as-Made?id=1553&name=Public%20Holidays%20(Ra%20o%20te%20Ui%20Ariki)%20Amendment%20Act%202011%20No%204)
        * [Public Holidays (Ra o te Ui Ariki) Amendment 2013](https://web.archive.org/web/20250530122050/https://parliament.gov.ck/wp-content/uploads/2020/02/Public-Holidays-Ra-o-te-Ui-Ariki-No.-4.pdf)
        * [2004](https://web.archive.org/web/20090326004400/http://www.cook-islands.gov.ck/view_release.php?release_id=429)
        * [2008](https://web.archive.org/web/20090326003301/http://www.stats.gov.ck/NewsEvents/PublicHolidays.pdf)
    """

    country = "CK"
    default_language = "en_CK"
    # %s (observed).
    observed_label = tr("%s (observed)")
    supported_languages = ("en_CK", "en_US")
    # Public Holiday Act 1999.
    start_year = 2000

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        self._add_observed(
            # New Year's Day.
            self._add_new_years_day(tr("New Year's Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        self._add_observed(
            # Day after New Year's Day.
            self._add_new_years_day_two(tr("Day after New Year's Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        # Anzac Day.
        self._add_anzac_day(tr("Anzac Day"))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Easter Monday.
        self._add_easter_monday(tr("Easter Monday"))

        # Sovereign's Birthday.
        self._add_holiday_1st_mon_of_jun(tr("Sovereign's Birthday"))

        if self._year >= 2012:
            # Day of the House of Ariki.
            name = tr("Ra o te Ui Ariki")
            if self._year >= 2013:
                self._add_holiday_1st_fri_of_jul(name)
            else:
                self._add_holiday_jun_6(name)

        # Constitution Day.
        self._add_observed(self._add_holiday_aug_4(tr("Constitution Day")))

        # Cook Islands Gospel Day.
        self._add_observed(self._add_holiday_oct_26(tr("Cook Islands Gospel Day")))

        self._add_observed(
            # Christmas Day.
            self._add_christmas_day(tr("Christmas Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        self._add_observed(
            # Boxing Day.
            self._add_christmas_day_two(tr("Boxing Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        if self._year <= 2011:
            # Penrhyn Gospel Day.
            self._add_observed(self._add_holiday_mar_13(tr("Penrhyn Gospel Day")))

            # Palmerston Gospel Day.
            self._add_observed(self._add_holiday_may_25(tr("Palmerston Gospel Day")))

            # Mangaia Gospel Day.
            self._add_observed(self._add_holiday_jun_15(tr("Mangaia Gospel Day")))

            # Atiu Gospel Day.
            self._add_observed(self._add_holiday_jul_20(tr("Atiu Gospel Day")))

            # Mitiaro Gospel Day.
            self._add_observed(self._add_holiday_jul_21(tr("Mitiaro Gospel Day")))

            self._add_observed(
                # Mauke Gospel Day.
                self._add_holiday_jul_23(tr("Mauke Gospel Day")),
                rule=SAT_TO_NEXT_TUE if self._year in {2005, 2011} else None,
            )

            # Rarotonga Gospel Day.
            self._add_observed(self._add_holiday_jul_25(tr("Rarotonga Gospel Day")))

            # Manihiki Gospel Day.
            self._add_observed(self._add_holiday_aug_8(tr("Manihiki Gospel Day")))

            # Rakahanga Gospel Day.
            self._add_observed(self._add_holiday_aug_15(tr("Rakahanga Gospel Day")))

            # Aitutaki Gospel Day.
            self._add_observed(self._add_holiday_oct_27(tr("Aitutaki Gospel Day")))

            # Pukapuka Gospel Day.
            self._add_observed(self._add_holiday_dec_8(tr("Pukapuka Gospel Day")))


class CK(CookIslands):
    pass


class COK(CookIslands):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.guinea import Guinea
from tests.common import CommonCountryTests


class TestGuinea(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Guinea)

    def test_new_years_day(self):
        name = "Fête du Nouvel an"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        dt = (
            "2023-01-02",
            "2034-01-02",
        )
        self.assertHolidayName(f"Lendemain de la {name}", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_second_republic_day(self):
        name = "Jour de la Deuxième République"
        self.assertHolidayName(name, (f"{year}-04-03" for year in range(1985, 2022)))
        self.assertNoHolidayName(name, range(self.start_year, 1985), range(2022, self.end_year))

    def test_easter_monday(self):
        name = "Lundi de Pâques"
        self.assertHolidayName(
            name,
            "2015-04-06",
            "2016-03-28",
            "2017-04-17",
            "2018-04-02",
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        self.assertHolidayName("Fête du Travail", (f"{year}-05-01" for year in self.full_range))

    def test_africa_day(self):
        self.assertHolidayName(
            "Anniversaire de l'Union Africaine", (f"{year}-05-25" for year in self.full_range)
        )

    def test_assumption_of_mary(self):
        self.assertHolidayName("Assomption", (f"{year}-08-15" for year in self.full_range))

    def test_independence_day(self):
        name = "Fête anniversaire de l'indépendance de la Guinée"
        self.assertHolidayName(name, (f"{year}-10-02" for year in self.full_range))
        dt = (
            "2033-10-03",
            "2039-10-03",
        )
        self.assertHolidayName(f"Lendemain de la {name}", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_all_saints_day(self):
        name = "Toussaint"
        self.assertHolidayName(name, (f"{year}-11-01" for year in range(self.start_year, 2022)))
        self.assertNoHolidayName(name, range(2022, self.end_year))

    def test_christmas_day(self):
        self.assertHolidayName("Fête de Noël", (f"{year}-12-25" for year in self.full_range))

    def test_laylat_al_qadr_day(self):
        name = "Lendemain de la nuit Lailatoul Qadr"
        self.assertHolidayName(
            name,
            "2015-07-14",
            "2016-07-03",
            "2017-06-22",
            "2018-06-12",
            "2019-06-01",
            "2020-05-20",
            "2021-05-09",
            "2022-04-29",
            "2023-04-18",
            "2024-04-06",
            "2025-03-27",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_fitr_day(self):
        name = "Jour de l'Aïd el-Fitr"
        self.assertHolidayName(
            name,
            "2015-07-18",
            "2016-07-07",
            "2017-06-26",
            "2018-06-15",
            "2019-06-04",
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-31",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        dt = (
            "2033-01-03",
            "2037-11-09",
        )
        self.assertHolidayName(f"Lendemain de la {name} (estimé)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_eid_al_adha_day(self):
        name = "Jour de la Tabaski"
        self.assertHolidayName(
            name,
            "2015-09-24",
            "2016-09-13",
            "2017-09-02",
            "2018-08-22",
            "2019-08-11",
            "2020-07-31",
            "2021-07-20",
            "2022-07-10",
            "2023-06-28",
            "2024-06-16",
            "2025-06-07",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha_day_two(self):
        name = "Lendemain de la Tabaski"
        self.assertHolidayName(
            name,
            "2023-06-29",
            "2024-06-17",
            "2025-06-08",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(2023, self.end_year))
        self.assertNoIslamicNoEstimatedHolidayName(name, range(self.start_year, 2023))

    def test_mawlid_day(self):
        name = "Lendemain de la nuit du Maoloud"
        self.assertHolidayName(
            name,
            "2015-12-24",
            "2016-12-12",
            "2017-12-01",
            "2018-11-20",
            "2019-11-10",
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
            "2025-09-05",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_2021(self):
        # https://www.timeanddate.com/holidays/guinea/2021
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "Fête du Nouvel an"),
            ("2021-04-03", "Jour de la Deuxième République"),
            ("2021-04-05", "Lundi de Pâques"),
            ("2021-05-01", "Fête du Travail"),
            ("2021-05-09", "Lendemain de la nuit Lailatoul Qadr"),
            ("2021-05-13", "Jour de l'Aïd el-Fitr"),
            ("2021-05-25", "Anniversaire de l'Union Africaine"),
            ("2021-07-20", "Jour de la Tabaski"),
            ("2021-08-15", "Assomption"),
            ("2021-10-02", "Fête anniversaire de l'indépendance de la Guinée"),
            ("2021-10-18", "Lendemain de la nuit du Maoloud"),
            ("2021-11-01", "Toussaint"),
            ("2021-12-25", "Fête de Noël"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Fête du Nouvel an"),
            ("2024-04-01", "Lundi de Pâques"),
            ("2024-04-06", "Lendemain de la nuit Lailatoul Qadr"),
            ("2024-04-10", "Jour de l'Aïd el-Fitr"),
            ("2024-05-01", "Fête du Travail"),
            ("2024-05-25", "Anniversaire de l'Union Africaine"),
            ("2024-06-16", "Jour de la Tabaski"),
            ("2024-06-17", "Lendemain de la Tabaski"),
            ("2024-08-15", "Assomption"),
            ("2024-09-15", "Lendemain de la nuit du Maoloud"),
            ("2024-10-02", "Fête anniversaire de l'indépendance de la Guinée"),
            ("2024-12-25", "Fête de Noël"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-04-01", "Easter Monday"),
            ("2024-04-06", "Day after Night of Power"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-05-01", "Labor Day"),
            ("2024-05-25", "Africa Day"),
            ("2024-06-16", "Eid al-Adha"),
            ("2024-06-17", "Day after Eid al-Adha"),
            ("2024-08-15", "Assumption Day"),
            ("2024-09-15", "Day after Prophet's Birthday"),
            ("2024-10-02", "Independence Day"),
            ("2024-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.tajikistan import Tajikistan
from tests.common import CommonCountryTests


class TestTajikistan(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(1992, 2050)
        super().setUpClass(Tajikistan, years=years)
        cls.no_estimated_holidays = Tajikistan(years=years, islamic_show_estimated=False)

    def test_new_years_day(self):
        self.assertHolidayName("Соли Нав", (f"{year}-01-01" for year in range(1992, 2050)))

    def test_mothers_day(self):
        self.assertHolidayName("Рӯзи Модар", (f"{year}-03-08" for year in range(1992, 2050)))

    def test_nowruz(self):
        name = "Иди байналмилалии Наврӯз"
        self.assertHolidayName(name, (f"{year}-03-21" for year in range(1992, 2050)))
        self.assertHolidayName(name, (f"{year}-03-22" for year in range(1992, 2050)))
        self.assertHolidayName(name, (f"{year}-03-23" for year in range(2003, 2050)))
        self.assertHolidayName(name, (f"{year}-03-24" for year in range(2006, 2050)))
        self.assertNoHolidayName(name, (f"{year}-03-23" for year in range(1992, 2003)))
        self.assertNoHolidayName(name, (f"{year}-03-24" for year in range(1992, 2006)))

    def test_workers_day(self):
        name = "Рӯзи байналхалқии якдилии меҳнаткашон"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1992, 2017)))
        self.assertNoHolidayName(name, range(2017, 2050))

    def test_victory_day(self):
        self.assertHolidayName(
            "Рӯзи Ғалаба дар Ҷанги Бузурги Ватанӣ", (f"{year}-05-09" for year in range(1992, 2050))
        )

    def test_unity_day(self):
        name = "Рӯзи Ваҳдати миллӣ"
        self.assertHolidayName(name, (f"{year}-06-27" for year in range(1998, 2050)))
        self.assertNoHolidayName(name, range(1992, 1998))

    def test_independence_day(self):
        self.assertHolidayName(
            "Рӯзи Истиқлолияти давлатии Ҷумҳурии Тоҷикистон",
            (f"{year}-09-09" for year in range(1992, 2050)),
        )

    def test_constitution_day(self):
        name = "Рӯзи Конститутсияи Ҷумҳурии Тоҷикистон"
        self.assertHolidayName(name, (f"{year}-11-06" for year in range(1994, 2050)))
        self.assertNoHolidayName(name, range(1992, 1994))

    def test_eid_al_fitr(self):
        name = "Рӯзи иди Рамазон"
        self.assertHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-22",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(1992, 2050))

    def test_eid_al_adha(self):
        name = "Рӯзи иди Қурбон"
        self.assertHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(1992, 2050))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Соли Нав"),
            ("2024-03-08", "Рӯзи Модар"),
            ("2024-03-21", "Иди байналмилалии Наврӯз"),
            ("2024-03-22", "Иди байналмилалии Наврӯз"),
            ("2024-03-23", "Иди байналмилалии Наврӯз"),
            ("2024-03-24", "Иди байналмилалии Наврӯз"),
            ("2024-04-10", "Рӯзи иди Рамазон"),
            ("2024-05-09", "Рӯзи Ғалаба дар Ҷанги Бузурги Ватанӣ"),
            ("2024-06-16", "Рӯзи иди Қурбон"),
            ("2024-06-27", "Рӯзи Ваҳдати миллӣ"),
            ("2024-09-09", "Рӯзи Истиқлолияти давлатии Ҷумҳурии Тоҷикистон"),
            ("2024-11-06", "Рӯзи Конститутсияи Ҷумҳурии Тоҷикистон"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-03-08", "Mother's Day"),
            ("2024-03-21", "International Nowruz Day"),
            ("2024-03-22", "International Nowruz Day"),
            ("2024-03-23", "International Nowruz Day"),
            ("2024-03-24", "International Nowruz Day"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-05-09", "Victory Day"),
            ("2024-06-16", "Eid al-Adha"),
            ("2024-06-27", "National Unity Day"),
            ("2024-09-09", "Independence Day"),
            ("2024-11-06", "Constitution Day"),
        )

    def test_l10n_ru(self):
        self.assertLocalizedHolidays(
            "ru",
            ("2024-01-01", "Новый год"),
            ("2024-03-08", "День Матери"),
            ("2024-03-21", "Международный праздник Навруз"),
            ("2024-03-22", "Международный праздник Навруз"),
            ("2024-03-23", "Международный праздник Навруз"),
            ("2024-03-24", "Международный праздник Навруз"),
            ("2024-04-10", "Ураза-байрам"),
            ("2024-05-09", "День Победы в Великой Отечественной войне"),
            ("2024-06-16", "Курбан-байрам"),
            ("2024-06-27", "День Национального единства"),
            ("2024-09-09", "День Государственной независимости Республики Таджикистан"),
            ("2024-11-06", "День Конституции Республики Таджикистан"),
        )
